/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/


/*
Sample client to illustrate the use of the QUANTAnet_datapack_c class.
*/

#include <QUANTA/QUANTA.hxx>
#include <stdio.h>

int main (int argc, char *argv[])
{
	int dataSize;
	char *receiveBuffer;

	char receivedChar;
	int receivedInt;
	long receivedLong;
    unsigned long receivedUnsignedLong;
	float receivedFloat;
	double receivedDouble;
	float* floatArray = new float[4];
    QUANTAnet_datapack_c packer;
	char receivedStr[8];

	QUANTAinit();
	QUANTAnet_tcpClient_c *client = new QUANTAnet_tcpClient_c;
	if (argc < 2){
		printf("Usage: %s machine_name\n",argv[0]);
		exit(1);
	}
	
	if (!client->connectToServer(argv[1], 9999)){
		printf("Can not connect to server\n");
		exit(1);
	}

	dataSize=packer.sizeof_char()+packer.sizeof_int()+packer.sizeof_float()+
    packer.sizeof_double() + packer.sizeof_long() + 
    packer.sizeof_unsignedLong() + packer.sizeof_char(8) + 
    packer.sizeof_float()*4;

	receiveBuffer=new char[dataSize];
	client->read(receiveBuffer, &dataSize,QUANTAnet_tcpClient_c::BLOCKING);
	packer.initUnpack(receiveBuffer,dataSize);
	packer.unpackFloat(&receivedFloat);
	packer.unpackInt(&receivedInt);
	packer.unpackDouble(&receivedDouble);
	packer.unpackChar(&receivedChar);
	packer.unpack(receivedStr,8);
	packer.unpackFloatArray(floatArray, 4);
	packer.unpackLong(&receivedLong);
    packer.unpackUnsignedLong(&receivedUnsignedLong);

	printf("Received Char: %c\n",receivedChar);
	printf("Received Int: %d\n",receivedInt);
	printf("Received Long: %ld\n",receivedLong);
    printf("Received Unsigned Long: %lu\n",receivedUnsignedLong);
	printf("Received Float: %f\n",receivedFloat);
	printf("Received Double: %f\n",receivedDouble);
	printf("Received String: %s\n",receivedStr);
    printf("Received Float Array - with the floats: ");
    for(int i=0; i<4; i++)
        printf("%f\t", floatArray[i]);
                   
	client->close();
	delete receiveBuffer;
	return 1;
}
