/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

/*
The server and client sample programs illustrate the use of QUANTAnet_datapack_c class. Miscellaneous data like floats, doubles, longs etc are packed in a single buffer and sent by the server across the network, to be read and unpacked by the client. The 

array packing and unpacking API's have also been used in these sample programs.
*/


#include <QUANTA/QUANTA.hxx>

int main (int argc, char *argv[])
{

	QUANTAnet_tcpServer_c *server;
	QUANTAnet_datapack_c packer;

	char mychar='A';
	int myint = 20000000;
	float myfloat = 3.33f;
    unsigned long myUnsignedLong = 4000000000;
	double mydouble = 4.44444;
	long mylong = 1138;
	char mystr[8];
    strcpy(mystr,"thx1138");
    float floatArray[4] = {1.1f, 2.2f, 3.3f, 4.4f};
	printf("char=%d\n",packer.sizeof_char());
	printf("int=%d\n",packer.sizeof_int());
	printf("float=%d\n",packer.sizeof_float());
	printf("double=%d\n",packer.sizeof_double());
	printf("long=%d\n",packer.sizeof_long());
    printf("unsigned long=%d\n",packer.sizeof_unsignedLong());

	int dataSize=packer.sizeof_char()+packer.sizeof_int()+packer.sizeof_float()+
    packer.sizeof_double() +  packer.sizeof_char(8) + packer.sizeof_long() + 
    packer.sizeof_unsignedLong() + packer.sizeof_float()*4;

	char *sendBuffer;
	sendBuffer = new char[dataSize];

	QUANTAinit();

	server = new QUANTAnet_tcpServer_c;
	if (server->init(9999) == 0)	{
		printf("Can not start server\n");
		exit(1);
	}


	QUANTAnet_tcpClient_c *aClient;
	aClient=NULL;
	while(NULL==aClient){
		aClient = server->checkForNewConnections();
		QUANTAsleep(1);

	}

	printf("Send Char: %c\n", mychar);
	printf("Send Int: %d\n", myint);
	printf("Send Long: %ld\n", mylong);
    printf("Send Unsigned Long: %lu\n", myUnsignedLong);
	printf("Send Float: %f\n", myfloat);
	printf("Send Double: %lf\n", mydouble);
	printf("Send String: %s\n", mystr);
	printf("Send Float Array - with the floats: ");
    for(int i=0; i<4; i++)
        printf("%f\t", floatArray[i]);
	packer.initPack(sendBuffer,dataSize);
	packer.packFloat(myfloat);
	packer.packInt(myint);
	packer.packDouble(mydouble);
	packer.packChar(mychar);
	packer.pack(mystr,8);
    packer.packFloatArray(floatArray, 4);
    packer.packLong(mylong);
    packer.packUnsignedLong(myUnsignedLong);
    int realSize = packer.getBufferFilledSize();
	printf("\n\nSIZE: %d/%d\n",realSize, dataSize);
	aClient->write(sendBuffer, &realSize, QUANTAnet_tcpClient_c::BLOCKING);

	aClient->close();
	server->close();
	delete sendBuffer;
	return 1;
}

