/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include <QUANTA/QUANTA.hxx>
#include <stdio.h>

int
main (int argc, char *argv[])
{
    const int SIZE = 4;
	int i;

	QUANTAinit();

        QUANTAnet_extendedParallelTcpServer_c* server = new QUANTAnet_extendedParallelTcpServer_c;
        if (server->init(9999) != QUANTAnet_parallelTcpServer_c::OK)    {
		printf("Can not start server\n");
		exit(1);
	}

	QUANTAnet_extendedParallelTcpClient_c *client;
	client = NULL;
	while(NULL == client){
		client = server->checkForNewConnections();
		QUANTAsleep(1);
	}

	char sendChar = 'c';
	printf("Send Char: %c\n", sendChar);
	client->writeChar(sendChar);

	int sendInt = 23493;
	printf("Send Int: %d\n", sendInt);
	client->writeInt(sendInt);

	int sendIntArray[SIZE] = { 3823, 2384, 3928, 3743 };
    printf("Send Int Array\n");
	for (i = 0; i < SIZE; i++) printf("array[%d] = %d\n", i, sendIntArray[i]);
	client->writeIntArray(sendIntArray, SIZE);

	int32 sendInt32 = 23493233;
	printf("Send Int32: %d\n", sendInt32);
	client->writeInt32(sendInt32);

	int32 sendInt32Array[SIZE] = { 32211, 384929, 293842, 293847 };
	client->writeInt32Array(sendInt32Array, SIZE);
	printf("Send Int32 Array\n");
	for (i = 0; i < SIZE; i++) printf("array[%d] = %d\n", i, sendInt32Array[i]);

#if (_MIPS_SZLONG==64) 
	int64 sendInt64 = 102039483920;
	client->writeInt64(sendInt64);
	printf("Send Int64: %ld\n", sendInt64);

	int64 sendInt64Array[SIZE] = { 100000000000, 200000000000, 30000000000, 40000000000 };
	client->writeInt64Array(sendInt64Array, SIZE);
	printf("Send Int64 Array\n");
	for (i = 0; i < SIZE; i++) printf("array[%d] = %ld\n", i, sendInt64Array[i]);
#endif

	long sendLong = 10000000;
	client->writeLong(sendLong);
	printf("Send Long: %ld\n",sendLong);

	long sendLongArray[SIZE] = { 1000000, 200000, 3000000, 4000000 };
	client->writeLongArray(sendLongArray, SIZE);
	printf("Send Long Array\n");
	for (i = 0; i < SIZE; i++) printf("array[%d] = %ld\n", i, sendLongArray[i]);

	float sendFloat = 38293.382f;
	client->writeFloat(sendFloat);
	printf("Send Float: %f\n",sendFloat);

	float sendFloatArray[SIZE] = { 328392.323f, 328374.322f, 328293.234f, 271839.433f };
	client->writeFloatArray(sendFloatArray, SIZE);
	printf("Send Float Array\n");
	for (i = 0; i < SIZE; i++) printf("array[%d] = %f\n", i, sendFloatArray[i]);

	double sendDouble = 3281919382.348;
	client->writeDouble(sendDouble);
	printf("Send Double: %f\n",sendDouble);

	double sendDoubleArray[SIZE] = { 38238392.383, 38493023.383, 12323458.392, 328192834.382 };
	client->writeDoubleArray(sendDoubleArray, SIZE);
	printf("Send Double Array\n");
	for (i = 0; i < SIZE; i++) printf("array[%d] = %lf\n", i, sendDoubleArray[i]);

	client->close();
	delete client;
	server->close();
	delete server;
	return 1;
}
