/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include <QUANTA/QUANTA.hxx>
#include <stdio.h>

int
main (int argc, char *argv[])
{
    const int SIZE = 4;
	int i;

	QUANTAinit();
	QUANTAnet_extendedTcpClient_c *client = new QUANTAnet_extendedTcpClient_c;
	if (argc < 2){
		printf("Usage: %s machine_IP_address\n",argv[0]);
		exit(1);
	}
	
        if (client->connectToServer(argv[1], 8566) <= 0) {
		printf("Cannot connect to server\n");
		exit(1);
	}

	char receivedChar;
	client->readChar(receivedChar, QUANTAnet_tcpClient_c::BLOCKING);
	printf("Received Char: %c\n", receivedChar);

	int receivedInt;
	client->readInt(receivedInt, QUANTAnet_tcpClient_c::BLOCKING);
	printf("Received Int: %d\n", receivedInt);

	int receivedIntArray[SIZE];
	client->readIntArray(receivedIntArray, SIZE, QUANTAnet_tcpClient_c::BLOCKING);
    printf("Received Int Array\n");
	for (i = 0; i < SIZE; i++) printf("array[%d] = %d\n", i, receivedIntArray[i]);

	int32 receivedInt32 = 23493233;
	printf("Received Int32: %d\n", receivedInt32);
	client->readInt32(receivedInt32, QUANTAnet_tcpClient_c::BLOCKING);

	int32 receivedInt32Array[SIZE];
	client->readInt32Array(receivedInt32Array, SIZE, QUANTAnet_tcpClient_c::BLOCKING);
	printf("Received Int32 Array\n");
	for (i = 0; i < SIZE; i++) printf("array[%d] = %d\n", i, receivedInt32Array[i]);

#if (_MIPS_SZLONG==64) 
	int64 receivedInt64 = 102039483920;
	client->readInt64(receivedInt64, QUANTAnet_tcpClient_c::BLOCKING);
	printf("Received Int64: %ld\n", receivedInt64);

	int64 receivedInt64Array[SIZE] = { 100000000000, 200000000000, 30000000000, 40000000000 };
	client->writeInt64Array(receivedInt64Array, SIZE, QUANTAnet_tcpClient_c::BLOCKING);
	printf("Received Int64 Array\n");
	for (i = 0; i < SIZE; i++) printf("array[%d] = %ld\n", i, receivedInt64Array[i]);
#endif

	long receivedLong;
	client->readLong(receivedLong, QUANTAnet_tcpClient_c::BLOCKING);
	printf("Received Long: %ld\n",receivedLong);

	long receivedLongArray[SIZE];
	client->readLongArray(receivedLongArray, SIZE, QUANTAnet_tcpClient_c::BLOCKING);
	printf("Received Long Array\n");
	for (i = 0; i < SIZE; i++) printf("array[%d] = %ld\n", i, receivedLongArray[i]);

	float receivedFloat;
	client->readFloat(receivedFloat, QUANTAnet_tcpClient_c::BLOCKING);
	printf("Received Float: %f\n",receivedFloat);

	float receivedFloatArray[SIZE];
	client->readFloatArray(receivedFloatArray, SIZE, QUANTAnet_tcpClient_c::BLOCKING);
	printf("Received Float Array\n");
	for (i = 0; i < SIZE; i++) printf("array[%d] = %f\n", i, receivedFloatArray[i]);

	double receivedDouble;
	client->readDouble(receivedDouble, QUANTAnet_tcpClient_c::BLOCKING);
	printf("Received Double: %f\n",receivedDouble);

	double receivedDoubleArray[SIZE];
	client->readDoubleArray(receivedDoubleArray, SIZE, QUANTAnet_tcpClient_c::BLOCKING);
	printf("Received Double Array\n");
	for (i = 0; i < SIZE; i++) printf("array[%d] = %lf\n", i, receivedDoubleArray[i]);
	return 1;
}
