/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <unistd.h>
#include <signal.h>

#include "QUANTAnet_fecServer_c.hxx"

QUANTAnet_fecServer_c* fecServer;
char* message;

void quit(int the_sig);

void myExportFunc(void * data, int size)
{
	// Verify the correctness of the data.
        char *ptr;
        ptr = (char *)data;
        printf("%d\n", *ptr);
}

int main(int argc, char** argv)
{
	
	unsigned int numSourcePack, numTotalPack, numRowofRecvBuff;
	unsigned short timeOut = 10 ;
	unsigned short srcPort, fecPort;
	int* count;
	int mStatus;
	int packetSize;
	
	if(argc < 7)
	{
		cerr << "server srcPort fecPort numSrcPack numTotalPack numRowofRecvBuff packetSize" << endl;
		exit(1);
	}
		
	srcPort = (unsigned short)atoi(argv[1]);
	fecPort = (unsigned short)atoi(argv[2]);
	numSourcePack = (unsigned int)atoi(argv[3]);
	numTotalPack = (unsigned int)atoi(argv[4]);
	numRowofRecvBuff = (unsigned int)atoi(argv[5]);
	packetSize = (unsigned int)atoi(argv[6]);
	
printf("source data UDP port: %d\n", srcPort);
printf("fec data UDP port: %d\n", fecPort);
printf("The ratio of src to fec: %d:%d\n", numSourcePack, numTotalPack-numSourcePack);
printf("Row of receiving buffer: %d\n", numRowofRecvBuff);
printf("UDP payload size: %d\n", packetSize);


	if(signal(SIGINT, quit) == SIG_ERR)
	{
	  perror("SIGINT");
	  exit(1);
	}
	
	fecServer = new QUANTAnet_fecServer_c();
	
	fecServer->init(numSourcePack,numTotalPack,packetSize,numRowofRecvBuff,timeOut,srcPort,fecPort);

	fecServer->streamingRecv(&myExportFunc);
	fecServer->printStatisticsData();
	return 0;
}

void quit(int the_sig)
{
	delete fecServer;
	delete message;
	exit(0);
} 
