/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include "QUANTAnet_fecClient_c.hxx"

QUANTAnet_fecClient_c* fecClient;
char * globaldata;
static int global_seq;

void quit(int the_sig);

void updateFunc(void * data, int size)
{
//	memcpy(data, globaldata, size);
  
	memset(data, global_seq, size);
	global_seq ++;
}

int main(int argc, char** argv)
{
	
	unsigned int numSourcePack, numTotalPack, numRowofRecvBuff;
	char* hostname;
	unsigned short srcPort, fecPort;
	int count = 0;
	double sendRate;
	int packetSize;
	
	if(argc < 8)
	{
		cerr << "client hostname srcPort fecPort numSrcPack numTotalPack sendRate packetSize" << endl;
		exit(1);
	}
		
	hostname = argv[1];
	srcPort = (unsigned short)atoi(argv[2]);
	fecPort = (unsigned short)atoi(argv[3]);
	numSourcePack = (unsigned int)atoi(argv[4]);
	numTotalPack = (unsigned int)atoi(argv[5]);
	sendRate = (double)atoi(argv[6]);
	packetSize = (int)atoi(argv[7]);
	
	if(signal(SIGINT, quit) == SIG_ERR)
	{
	  perror("SIGINT");
	  exit(1);
	}
	
	fecClient = new QUANTAnet_fecClient_c();
	
printf("Receiver: %s\n", hostname);
printf("source data UDP port: %d\n", srcPort);
printf("fec data UDP port: %d\n", fecPort);
printf("The ratio of src to fec: %d:%d\n", numSourcePack, numTotalPack-numSourcePack);
printf("Sending Rate: %d\n", sendRate);
printf("UDP payload size: %d\n", packetSize);
	fecClient->init(numSourcePack,numTotalPack,packetSize,hostname,srcPort,fecPort);

	global_seq = 0;
	globaldata = (char *)malloc(packetSize*numSourcePack);
	printf("starting steaming data\n");
	// set the raw data rate, the real traffic should be more.
	fecClient->setSendRate(sendRate);
	fecClient->streamingSend(&updateFunc);
	return 0;
		
}

void quit(int the_sig)
{
	delete fecClient;
	exit(0);
}
	
