/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

/* Demo for mcast class.
 * Compile this with: make newclient
 *
 * On 2 separate machines type:
 * newclient 225.0.0.37 7777 hello
 * newclient 225.0.0.37 7777 there
 * Now watch the messages fly.
 *
 */

#include <QUANTA/QUANTA.hxx>
#ifdef WIN32
#include <winsock.h>
#else
#include <sys/errno.h>
#endif
#include <stdlib.h>

#define BUFSIZE 256

#ifdef WIN32
#define EWOULDBLOCK WSAEWOULDBLOCK 
#endif

int main(int argc, char **argv)
{

	if (argc < 2) {
		printf("Usage: %s mcast_host (e.g. 225.0.0.37) mcast_port (e.g. # > 6000) <string>\n",argv[0]);
		exit(1);
	}

	QUANTAinit();

	/* Make an mcast object */
	QUANTAnet_mcast_c *client;

	client = new QUANTAnet_mcast_c;

	char selfIP[256];
	client->getSelfIP(selfIP);
	printf("I am: %s %lu\n",selfIP,client->getSelfIP());
	

	/* Open to mcast host and port */
	if (client->init(argv[1],atoi(argv[2])) == QUANTAnet_mcast_c::FAILED) {
		printf("Connection error. Bailing out\n"); exit(1);
	}

	/* Make it a non blocking connection */
	client->makeNonBlocking();

	char buffer[BUFSIZE];
	char outMsg[BUFSIZE];
	int counter = 0;
	while(1) {

		/* Try and receive something */
		if (client->receive(buffer, BUFSIZE) < 0) {
			if (errno != EWOULDBLOCK)
			printf("Receive Error #%d\n", errno);
		}

		/* Send your command line string */
		sprintf(outMsg,"%s %d\n", argv[3], counter++);
		if (client->send(outMsg,strlen(outMsg)+1) < 0) {
			printf("Send Error #%d\n", errno);
		}

		printf("MESSAGE: %s\n",buffer);
		client->printInfo(); /* Print connection info */

		QUANTAsleep(1);
	}
        return 1;
}
