/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include <QUANTA/QUANTA.hxx>
#include "TestObj.hxx"


int main(int argc, char *argv[])
{
	TestObj *obj;
	QUANTAnet_objectInputStreamTcp_c *stream;
	
	QUANTAinit();

	// create a new stream object with a null network connection
	stream = new QUANTAnet_objectInputStreamTcp_c(9999);
	
	// loop until a client connects
	while(stream->getNetworkClient() == NULL)
	{
		// assign the network connection to the object stream
		stream->setNetworkClient(stream->getNetworkServer()->checkForNewConnections());
		QUANTAusleep(10000); // sleep this process to not take up too much cpu resource
	}
	
////////////////////////////////////////////////////////////////////////////////
	printf("client connected\n");
	
	// read the object from the stream and print the results 
	obj = new TestObj();
	stream->readObject(obj);
	printf("Received data: %d\n", obj->getData());

	// shutdown
	delete obj;
	stream->close();
	QUANTAexit();
	
	return 1;
}
