/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include <QUANTA/QUANTA.hxx> 

#ifndef WIN32
#include <iostream.h>
#include <sys/time.h>
#else
// QUANTA defines std namespace and uses STL.
#include <iostream>
#include <sys/timeb.h>

// Simplified version of gettimeofday function.
static int gettimeofday(struct timeval *tp, void *tzp)
{
    struct _timeb t;

    _ftime(&t);
    tp->tv_sec = t.time;
    tp->tv_usec = t.millitm * 1000;
    return 0;
}
#endif

int main( int a_iArgc, char *a_pcArgv[] )
{
  if( a_iArgc != 3 )
    {
      printf("usage: %s <port> <recv buffer size>\n ",a_pcArgv[0]);
      exit(1);
    }
  int iPort = atoi( a_pcArgv[1] );
  int iBufferSize = atoi( a_pcArgv[2] );

  int nTotalBytesReceived=0;
  char *pcBuffer = new char[iBufferSize];
  struct timeval tStartTime, tEndTime;
  float fElapsed=0;

  QUANTAinit();

  memset( pcBuffer, 0, iBufferSize );

  QUANTAnet_parallelTcpClient_c *client;
  QUANTAnet_parallelTcpServer_c uServer;

  uServer.init(iPort);

  client = uServer.checkForNewConnections();
  gettimeofday(&tStartTime, NULL);

  memset( pcBuffer, 0, iBufferSize ); 
  int j=0;
  while (1)
  {
    j++;
    client->read( pcBuffer, &iBufferSize );
    printf("\nReceive -loop no:%d",j); 
  }
  nTotalBytesReceived+=iBufferSize;

  cout << endl;
  //printf("Total bytes received: %d\n",nTotalBytesReceived);

  gettimeofday(&tEndTime, NULL);

  fElapsed+=tEndTime.tv_sec-tStartTime.tv_sec;
  fElapsed+=(float)((float)(tEndTime.tv_usec-tStartTime.tv_usec)/1000000);

  //printf("Approx Elapsed Seconds = %f\n",fElapsed);
  cout << "Done!" << endl;

  delete client;
  delete[] pcBuffer;
  return 1;
}

  
