/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include "QUANTAnet_rbudpReceiver_c.hxx"

struct rio_request 
{
  struct in_addr ip;
};

int main(int argc, char ** argv)
{
	char * buf;
	int size,i;
	struct timeval start, end;
	double throughput;
	int sockfd, udpport, server_port;
	struct sockaddr_in servaddr;
	rio_request request;

        if (argc < 3)
        {
                printf("Usage: dataclient <ServerIP> <ClientIP> \n");
                exit(1);
        }
       
	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    		perror("socket failed");
    		exit(1);
  	}

	server_port = 38000;
	bzero(&servaddr, sizeof(servaddr));
	servaddr.sin_family = AF_INET;
	servaddr.sin_port = htons(server_port);
	inet_pton(AF_INET, argv[1], &servaddr.sin_addr);

	connect(sockfd, (struct sockaddr *)& servaddr, sizeof(servaddr));

	sleep(1);

	inet_pton(AF_INET, argv[2], &request.ip);
	write(sockfd, &request, sizeof(request)); 

	read(sockfd, &udpport, sizeof(udpport));
	printf("The server assigned UDP port %d.\n", udpport);
	write(sockfd, &udpport, sizeof(udpport));
	
	size = 300000000;
	buf = (char *) malloc(size);
	if (buf == NULL)
	{
                printf("memory allocation fail.\n");
                exit(1);
        }

	QUANTAnet_rbudpReceiver_c * myreceiver = new QUANTAnet_rbudpReceiver_c(sockfd, udpport);
	myreceiver->init(argv[1]);
	
	printf("init finishes\n");
	gettimeofday(&start, NULL);
	myreceiver->receive(buf, size, 1452);
	gettimeofday(&end, NULL);

	// verify if the protocol transferred the exact data	
	for (i=0;i<size/4;i++)
		if (*(int*)(buf+i*4) != i)
		{
			printf("Verification failed on %d: %d!\n",i, *(int*)(buf+i*4));
			exit(1);
		}
	
	throughput = (double)size*8.0/(double)USEC(&start, &end);
	printf("throughtput: %f Mbps\n", throughput);

	printf("Please wait for quitting of the server, then press any key to quit.\n");	
	getchar();
	myreceiver->close();
	
	close(sockfd);

	return 0;
}	

