/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/
#include "QUANTAnet_rbudpSender_c.hxx"

/* client now caches its sockets to connect with server and reuses
 * whenever possible
 */

static struct in_addr Clientsockaddr[FD_SETSIZE];
static int Clientsock[FD_SETSIZE];
static int Nclientsockets= 0;
const int MAX_CLIENT= 256;

struct rio_request
{
  struct in_addr ip;
};

/*----------------------------------------------------------------
 * This is the server main .... 
 */

int main() {

//  Rioheader_t riohdr;
//  Riostatus_t riostatus;

  int server_port;
  int reply;
  int client[FD_SETSIZE];
  int s, sNew;
  struct sockaddr_in serverSA, from, origin;
  int len, n;
  int i,ierr;
  char buff[1024];
  int size, sendRate, udpport;
  char *buf;
  struct rio_request request;
  int udpPortPool[MAX_CLIENT];

  struct timeval tv;
  fd_set set;
  unsigned naps;

  /* select stuff */
  fd_set allset, readyset, excepts;
  int maxfd, maxi, nready;

  int ifileerr, nbytes;

  for (i=0;i<MAX_CLIENT;i++)
	udpPortPool[MAX_CLIENT] = 0;

  // RBUDP initialization.
  size = 300000000;
  sendRate = 600000;
  buf = (char *) malloc(size);
  if (buf == NULL)
  {
    printf("memory allocation failed.\n");
    exit(-1);
  }

  // Initialize the buffer content in order to verify the protocol.
  for (i=0; i<size/4; i++)
    *(int *)(buf+i*4) = i;

  server_port = 38000;
  /* Initialize server communication */
  if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    perror("socket failed");
    exit(1);
  }

  bzero(&serverSA, sizeof(serverSA));
  serverSA.sin_family = AF_INET;
  serverSA.sin_addr.s_addr = htonl(INADDR_ANY);
  serverSA.sin_port = htons(server_port);

  if (bind(s, (struct sockaddr *)&serverSA, 
	   sizeof(struct sockaddr_in))!= 0) {
    perror("Bind failed\n");
    close (s);
    exit(1);
  }

  listen (s, 5);
  len=sizeof(from);

  FD_ZERO(&allset);
  FD_ZERO(&readyset);

  FD_SET(s,&allset);

  /* clear list of client sockets */
  for(i=0; i<FD_SETSIZE; ++i) client[i]= -1;

  maxfd= s;
  maxi= 0;

  do {
    memcpy( &readyset, &allset, sizeof(allset) );

    nready= select(maxfd+1, &readyset, NULL,&excepts,NULL);
    if ( nready == -1 ) {
	    printf("select error, aborting\n");
      exit(-1);
    }

    /* since no timeout, should get something */
    if ( nready == 0 ) {
	    printf("nil set returned from select, aborting\n");
      exit(-1);;
    }

    /* New connection? */
    if (FD_ISSET(s,&readyset) ) {

      sNew = accept(s,  (struct sockaddr *)&from, (socklen_t*)&len);

      printf("A new client comes from %s, port %d\n",
	inet_ntop(AF_INET, &from.sin_addr, buff, sizeof(buff)),
	ntohs(from.sin_port));

      for(i=0; i<FD_SETSIZE; ++i ) 
	if ( client[i]<0 ) {
	  client[i]= sNew;
	  break;
	}
      if ( i== FD_SETSIZE ) {
	fprintf(stderr,"too many clients!\n"); 
	exit(-1);
      }

      FD_SET(sNew, &allset);  

      if ( sNew > maxfd ) maxfd= sNew;
      if ( i>maxi ) maxi= i;

      if ( --nready ) continue;
    }

    /* process reads; these are rio requests */

    for(i=0; i<=maxi; ++i ) {
      if ((sNew=client[i])<0 ) continue;

      if (FD_ISSET(sNew,&excepts)) {
	  /* assume SIGPIPE or worse, close connection */
	      printf("exception, closing\n");
	  FD_CLR(sNew,&allset);
	  client[i]= -1;
	  shutdown(sNew,1);
	  close(sNew);
	  continue;
      }

      if (FD_ISSET(sNew,&readyset)) {

	ierr = read(sNew, &request, sizeof(request)); 
	if ( ierr <= 0 ) {

		printf("closing client socket %d\n", sNew );

	  /* closed */
	  FD_CLR(sNew,&allset);
	  client[i]= -1;
	  shutdown(sNew,1);
	  close(sNew);

	} else {
	  printf("The client %s is requesting data\n", 
		inet_ntop(AF_INET, (void *)&request.ip, buff, sizeof(buff)));

	  // Negotiate a UDP port.
	  int j=0;
	  while(udpPortPool[j] != 0)
	  {
	    j++;
	    if (j == MAX_CLIENT) 
	    {
	      printf("Too many clients.\n");
	      exit(1);
	    }
	  }
	  udpPortPool[j] = 1;
	  j+=38000;
	  printf("Assign UDP port %d.\n", j);

	  write(sNew, &j, sizeof(j));
	  read(sNew, &udpport, sizeof(udpport));
	  if (udpport != j)
		printf("returned port is incorrect!\n");

	  printf("ready to send data...\n");

	  // Use RBUDP to send data.

          QUANTAnet_rbudpSender_c *mysender = new QUANTAnet_rbudpSender_c(sNew, udpport);
	  mysender->init(buff);

	  mysender->send(buf, size, sendRate, 1452);
	  udpPortPool[j-38000] = 0;
          
	}

	if ( --nready <= 0 ) break;
      } /* ISSET */

    } /* all seen clients */

  } while(1);

  return 0;
}
