/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include "QUANTAnet_rbudpReceiver_c.hxx"

int main(int argc, char ** argv)
{
	char * buf;
	int size,i;
	struct timeval start, end;
	double throughput;

        if (argc < 3)
        {
                printf("Usage: urecv <sender> <size of memory (Bytes)>\n");
                exit(1);
        }
        
	size = atoi(argv[2]);
	buf = (char *) malloc(size);
	if (buf == NULL)
	{
                printf("memory allocation fail.\n");
                exit(1);
        }

	QUANTAnet_rbudpReceiver_c * myreceiver = new QUANTAnet_rbudpReceiver_c(38000);
	myreceiver->init(argv[1]);
	
	printf("init finishes\n");
	gettimeofday(&start, NULL);
	myreceiver->receive(buf, size, 8800);
	gettimeofday(&end, NULL);
	myreceiver->close();

	// verify if the protocol transferred the exact data	
	for (i=0;i<size/4;i++)
		if (*(int*)(buf+i*4) != i)
		{
			printf("Verification failed on %d: %d!\n",i, *(int*)(buf+i*4));
			exit(1);
		}
	
	throughput = (double)size*8.0/(double)USEC(&start, &end);
	printf("throughtput: %f Mbps\n", throughput);
	return 1;
}	

