/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include <QUANTA/QUANTA.hxx>
#include <string.h>
#ifdef WIN32
//#define timeb _timeb
//#define ftime _ftime
#include <sys\timeb.h>
#elif __APPLE__
#include <sys/time.h>
#elif __FreeBSD__
#include <sys/types.h>
#include <sys/timeb.h>
#else
#include <sys/timeb.h>
#endif


int main(int argc, char** argv)
{
	QUANTAinit();

	QUANTAnet_remoteFileIOClient32_c client;
	if (argc <3)
        {            
            printf("\nInsufficient arguments: \nUsage:\nDownload remote file:\tclient <hostname> -d localfilename remotefilename\n\nRemote File List:\tclient <hostname> -f remotePathname -r(optional)\n");
            exit(1);
        }
    if (!strcmp(argv[2], "-f"))     //get remote file list
        {           
         #if (defined(__sgi)|| defined(linux) || defined(sun))
            QUANTAmisc_fileInfo_c* fileList;
            fileList=NULL;
            int* noObjects = new int;
            int doRecursion = QUANTAmisc_fileList_c::SKIP_RECURSION;
            if (argc ==5 )
            {
                if (!strcmp(argv[4], "-r"))
                {
                    printf("\nA recursive list of files are:\n");
                    doRecursion = QUANTAmisc_fileList_c::LIST_RECURSIVELY;
                }
             }  
             
            int result = client.getRemoteFileList(argv[1], argv[3], fileList, noObjects, doRecursion, QUANTAmisc_fileList_c::TRAVERSE_LINKS); 
            if (result== QUANTAnet_remoteFileIOClient32_c::FILE_ERROR)
                { 
                    printf("\nError reading file on the remote machine - check the path name\n");
                    exit(1);
                }
            if (result== QUANTAnet_remoteFileIOClient32_c::CANNOT_CONNECT)
                {
                 printf("\nConnection to server refused  please check the hostname and port \n");
                 exit(1);
                }
            if (result== QUANTAnet_remoteFileIOClient32_c::MEM_ALLOC_ERR)
            {
                printf("\nMemory Allocation Error -file list maybe too big. Please try a non recursive list\n");
                exit(1);
            }
            printf("\nThe remote path name is: %s\n", argv[3]); 
            printf("\nThe number of files = %d\n", *noObjects);
            if (fileList)
            {
                for (int i=0; i< *noObjects; i++)    
                    fileList[i].displayFileInfo();
                printf("\n");
            }
           
        #endif
        
        } //End of getting remote file list

        if (!strcmp(argv[2], "-d"))     //download
        {    
            int day, mon, year, hour, min, sec;
	        int day1, mon1, year1, hour1, min1, sec1;

	        client.getRemoteDateTime(argv[1], argv[4], &day, &mon, &year, &hour, &min, &sec);

	        printf("Remote file %s date/time stamp: %02d/%02d/%02d %02d:%02d:%02d\n", argv[4], mon, day, year, hour, min, sec);

#if (defined(__APPLE__) || defined(__FreeBSD__))
                struct timeval tStartTime, tEndTime;
                gettimeofday(&tStartTime, NULL);
#else
                struct timeb tStartTime, tEndTime;
                ftime(&tStartTime);
#endif /* __APPLE___ || __FreeBSD__ */
	        client.download(argv[1], argv[3], argv[4]);
#if (defined(__APPLE__) || defined(__FreeBSD__))
                gettimeofday(&tEndTime, NULL);    
#else
	        ftime(&tEndTime);
#endif /* __APPLE___ || __FreeBSD__ */

#if (defined(__APPLE__) || defined(__FreeBSD__))
               float fEndTime = tEndTime.tv_sec + tEndTime.tv_usec*0.000001f;
               float fStartTime = tStartTime.tv_sec + tStartTime.tv_usec*0.000001f;
               float fElapsed = fEndTime - fStartTime;
#else
            float fElapsed = tEndTime.time - tStartTime.time;
	        fElapsed += (float) ((float) (tEndTime.millitm - tStartTime.millitm) / 1000);
#endif
	        printf("Approximate download time in seconds = %f\n", fElapsed);
	        printf("\nDisplayig instantaneous statistics\n");
	        client.showStats("RemoteFileIOClient32_download_monitored", "some_local_data");
	        client.getLocalDateTime(argv[3], &day1, &mon1, &year1, &hour1, &min1, &sec1);

	        printf("Local file %s date/time stamp: %02d/%02d/%02d %02d:%02d:%02d\n", argv[3], mon1, day1, year1, hour1, min1, sec1);

	        switch(client.compareDateTime(day,mon,year,hour,min,sec,day1,mon1,year1,hour1,min1,sec1)) {
	        case 8: //QUANTAnet_remoteFileIOClient32_c::LATER:
	        printf("%02d/%02d/%02d %02d:%02d:%02d is later than %02d/%02d/%02d %02d:%02d:%02d\n", mon, day, year, hour, min, sec, mon1, day1, year1, hour1, min1, sec1);
		    break;

	        case 7: //QUANTAnet_remoteFileIOClient32_c::EARLIER:
		    printf("%02d/%02d/%02d %02d:%02d:%02d is earlier than %02d/%02d/%02d %02d:%02d:%02d\n", mon, day, year, hour, min, sec, mon1, day1, year1, hour1, min1, sec1);
		    break;

	        case 9: //QUANTAnet_remoteFileIOClient32_c::SAME:
		    printf("%02d/%02d/%02d %02d:%02d:%02d is same as %02d/%02d/%02d %02d:%02d:%02d\n", mon, day, year, hour, min, sec, mon1, day1, year1, hour1, min1, sec1);
		    break;
	    }
    }
    
/*    FILE* filePtr;
    filePtr = fopen("remoteFileIO32.log", "a");
    if (filePtr)
	{
        client.logStats("RemoteFileIOClient32_monitoring", "some_local_data",filePtr);
	}
    else
        printf("\nError opening logfile\n");
    fclose(filePtr);*/
    return 1;
}
