/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include "QUANTA.hxx"

#ifdef WIN32
#define timeb _timeb
#define ftime _ftime

#include <sys\timeb.h>
#else
#include <sys/timeb.h>
#endif


int main(int argc, char** argv)
{
	QUANTAinit();

	QUANTAnet_remoteFileIOClient64_c client;
	
	int day, mon, year, hour, min, sec;
	int day1, mon1, year1, hour1, min1, sec1;

	client.getRemoteDateTime(argv[1], argv[3], &day, &mon, &year, &hour, &min, &sec);

	printf("Remote file %s date/time stamp: %02d/%02d/%02d %02d:%02d:%02d\n", argv[3], mon, day, year, hour, min, sec);

        struct timeb tStartTime, tEndTime;
        ftime(&tStartTime);
	client.download(argv[1], argv[2], argv[3]);
	ftime(&tEndTime);

        float fElapsed = tEndTime.time - tStartTime.time;
	fElapsed += (float) ((float) (tEndTime.millitm - tStartTime.millitm) / 1000);

	printf("Approximate download time in seconds = %f\n", fElapsed);

	printf("\nDisplayig instantaneous statistics\n");
	client.showStats("RemoteFileIOClient64_monitoring", "some_local_data");

	client.getLocalDateTime(argv[2], &day1, &mon1, &year1, &hour1, &min1, &sec1);

	printf("Local file %s date/time stamp: %02d/%02d/%02d %02d:%02d:%02d\n", argv[2], mon1, day1, year1, hour1, min1, sec1);

	switch(client.compareDateTime(day,mon,year,hour,min,sec,day1,mon1,year1,hour1,min1,sec1)) {
	case 8: /*QUANTAnet_remoteFileIOClient64_c::LATER:*/
		printf("%02d/%02d/%02d %02d:%02d:%02d is later than %02d/%02d/%02d %02d:%02d:%02d\n", mon, day, year, hour, min, sec, mon1, day1, year1, hour1, min1, sec1);
		break;

	case 7: /*QUANTAnet_remoteFileIOClient64_c::EARLIER:*/
		printf("%02d/%02d/%02d %02d:%02d:%02d is earlier than %02d/%02d/%02d %02d:%02d:%02d\n", mon, day, year, hour, min, sec, mon1, day1, year1, hour1, min1, sec1);
		break;

	case 9: /*QUANTAnet_remoteFileIOClient64_c::SAME:*/
		printf("%02d/%02d/%02d %02d:%02d:%02d is same as %02d/%02d/%02d %02d:%02d:%02d\n", mon, day, year, hour, min, sec, mon1, day1, year1, hour1, min1, sec1);
		break;
	}
    FILE* filePtr;
    filePtr = fopen("remoteFileIO64.log", "a");
    if (filePtr)
	{
        client.logStats("RemoteFileIOClient64_monitoring", "some_local_data",filePtr);
	}
    else
        printf("\nError opening logfile\n");
    fclose(filePtr);
    return 1;
}
