/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include <QUANTA/QUANTA.hxx>

class Cat : public QUANTAts_thread_c
{
	public:
		Cat(char *host="localhost", unsigned int port=9753, const char*l="cat")
			: QUANTAts_thread_c() { _host = host; _port = port; _label = l; create(); }
		virtual ~Cat(void) { fprintf(stdout, "Cat::~Cat() - Cat destroyed\n");  /*delete node;*/ }

	protected:
		virtual void run(void)
		{
			_node = new QUANTAnet_barrierNodeTcp_c(_host, _port);
			while(true)
			{
				while(isRunning())
				{
					fprintf(stdout, "%s entering barrier\n", _label);
					_node->wait("test");
					fprintf(stdout, "%s leaving barrier\n", _label);
					QUANTAsleep(3);
				}
				
				QUANTAsleep(1);
			}
		}

	private:
		QUANTAnet_barrierNodeTcp_c *_node;
		unsigned int _port;
		char *_host;
		const char *_label;

};


class Mouse : public QUANTAts_thread_c
{
	public:
		Mouse(char *host="localhost", unsigned int port=9753, const char*l="mouse")
			: QUANTAts_thread_c() { _host = host; _port = port; _label = l; create(); }
		virtual ~Mouse(void) { fprintf(stdout, "Mouse::~Mouse() - Mouse destroyed\n"); /*delete node;*/ }

	protected:
		virtual void run(void)
		{
			_node = new QUANTAnet_barrierNodeTcp_c(_host, _port);
			while(true)
			{
				while(isRunning())
				{
					fprintf(stdout, "%s entering barrier\n", _label);
					_node->wait("test");
					fprintf(stdout, "%s leaving barrier\n", _label);
					QUANTAsleep(2);
				}
				
				QUANTAsleep(1);
			}
		}

	private:
		QUANTAnet_barrierNodeTcp_c *_node;
		unsigned int _port;
		char *_host;
		const char *_label;

};


int main(int argc, char *argv[])
{
	unsigned int port = 9753;
	char *host = "localhost";
	Cat *cat;
	Mouse *mouse;


	// parse command line for host and port options
	for(int i = 0; i < argc; i++) 
	{
		if(*argv[i]++ == '-') 
		{
			if(*argv[i] == 'p')
				port = atoi(argv[++i]);
			else if(*argv[i] == 'h')
				host = argv[++i];
			else
				fprintf(stderr, "%s command not supported\n", argv[i]);
		}
	}

	QUANTAinit();
	cat = new Cat(host, port);
	mouse = new Mouse(host, port);

	while(true)
		QUANTAsleep(1);

	QUANTAexit();
	return 1;
}
