/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

/* This class shows the use of QUANTAnet_iovec_c class 
 *
 * The client send and receive using writev(),readv() method in QUANTAnet_tcpClient_c 
 * class using QUANTAnet_iovec_c object. 
 * On every 20 second interval, it will send the object again to the server
 * The server will return "server alive" message 
 */

#include <stdlib.h>
#ifdef WIN32
#include <string.h>
#else
#include <strings.h>
#endif

#include <QUANTA/QUANTA.hxx>
#include <QUANTA/QUANTAnet_iovec_c.hxx>

int main (int argc, char *argv[])
{
	/* check command line args */
	if(argc < 2) 
	{
		printf("usage : %s <server>\n", argv[0]);
		exit(1);
	}
	
	int status, dataSize;

	//Four data structures to be added in iovec
	//used in sending data to server
	int    mesg1[5] = {1,2,3,4,5}; 
	float  mesg2[5] = {1.1, 2.2, 3.3, 4.4, 5.5}; 
	double mesg3[5] = {1.1, 2.2, 3.3, 4.4, 5.5}; 
	char   mesg4[6] =""; mesg4[6]='\0';	
	
	sprintf(mesg4,"cavern");

	//variables used in receiving data from server
	char   mesg5[6] =""; mesg5[6]='\0';	
	char   mesg6[5] =""; mesg6[5]='\0';	
	
	//create an iovec array which will contain 2 iovec structure
	QUANTAnet_iovec_c cl_send_iovec(4);
	QUANTAnet_iovec_c cl_recv_iovec(2);
	
	
	QUANTAinit();

	QUANTAnet_tcpClient_c *client = new QUANTAnet_tcpClient_c;
	
	if (client->connectToServer(argv[1], 6544) < 0)
	{
		printf("ioclienttcp :: could not connect to the server\n");
		exit(1);
	}

	while(1) 
	{
		char name[256];
		client->getSelfIP(name);
		printf("Client IP %s, Port: %u\n", name, client->getSelfPort());

		//Add the four data structures to iovec
		cl_send_iovec.add(mesg4, strlen(mesg4));
		cl_send_iovec.add(mesg1, sizeof(mesg1));
		cl_send_iovec.add(mesg2, sizeof(mesg2));
		cl_send_iovec.add(mesg3, sizeof(mesg3));
		
		printf("testing linux >>>>>\n");

		//send the iovec to server using writev() method
		status = client->writev(cl_send_iovec, QUANTAnet_tcpClient_c::BLOCKING);

		//clear the iovec array to add new values
		cl_send_iovec.removeAll();

		//if there is some error while writing
		if (status == QUANTAnet_tcpClient_c::OK) 
		{
			printf("\nSent data to Server\n");
		}
		else
		{
			client->showStatus(status, dataSize);
			exit(1);
		}

		cl_recv_iovec.add(mesg5, sizeof(mesg5));
		cl_recv_iovec.add(mesg6, sizeof(mesg6));
		
		//read the data from the server
		status =  client->readv(cl_recv_iovec, QUANTAnet_tcpClient_c::BLOCKING);
										
		//if status is OK
		if (status == QUANTAnet_tcpClient_c::OK) 
		{
			printf("\nMessage from server >>>>>> %s %s >>>>> \n",mesg5, mesg6);

            printf("\nPerformance Monitoring Statistics:\n");
            client->showStats("tcpClient_perf_monitoring", "Some_local_data");          
		} 
		//if there is some error while reading
		else if (status != QUANTAnet_tcpClient_c::NON_BLOCKING_HAS_NO_DATA)
		{
			client->showStatus(status, dataSize);
			exit(1);
		}

		//clear the iovec array to add new values
		cl_recv_iovec.removeAll();

		//wait for 20 seconds
		QUANTAsleep(20);
	}

    return 1;
}
