/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

/* This class shows the use of QUANTAnet_iovec_c class 
 *
 * This server can accept maximum 5 client connections.
 * It receives QUANTAnet_iovec from the clients
 * It sends "server alive" message to all the clients 
 */

#include <stdlib.h>
#ifdef WIN32
#include <string.h>
#else
#include <strings.h>
#endif

#include <QUANTA/QUANTA.hxx>
#include <QUANTA/QUANTAnet_iovec_c.hxx>

#define MAX_NUM_CLIENTS 5

int main (int argc, char *argv[])
{		

	int numberOfClients = 0;

	QUANTAnet_tcpClient_c *client[MAX_NUM_CLIENTS];

	QUANTAnet_tcpServer_c *server;
		
	int    j; //loop counter
	char   name[256];
	int    status, dataSize;

	//These variable are used to read data from QUANTAnet_iovec_c
	int    mesg1[5]; 
	float  mesg2[5]; 
	double mesg3[5]; 
	char   mesg4[6] = ""; mesg4[6]='\0';

	//variables used in sending data to clients
	char   mesg5[6] =""; mesg5[6]='\0';	
	char   mesg6[5] =""; mesg6[5]='\0';	
	
	sprintf(mesg5,"Server");
	sprintf(mesg6,"Alive");

	//create a QUANTAnet_iovec_c object which will contain 4 iovec structure
	//as you have 4 varaible to read
	//this object is used for receving data from cleints
	QUANTAnet_iovec_c sr_recv_iovec(4);

	//this object is used for sending data to clients
	QUANTAnet_iovec_c sr_send_iovec(2);

	QUANTAnet_tcpClient_c *aClient;	

	QUANTAinit();	

	server = new QUANTAnet_tcpServer_c;
	
	if (server->init(6544) == 0)	
	{
		exit(1);
	}

	while(1)
	{						
		//checks for new clients
		//returns NULL if there is no client connected
		aClient = server->checkForNewConnections(10);
		
		//if there is a client
		if (aClient) 
		{
			printf("\n>>> Client %d connected", numberOfClients); fflush(stdout);
			aClient->getSelfIP(name);
			printf(">>> %s,%u\n", name, aClient->getSelfPort());
			client[numberOfClients] = aClient;
			numberOfClients++;
		}	

		//receive data from all the clients
		for (int i = 0; i < numberOfClients; i++) 
		{					
			//Add to iovec before reading 
			//otherwise readv() method gives error
			//mesg1, mesg2, mesg3, mesg4 will be autoamtically updated after readv()
			sr_recv_iovec.add(mesg4, sizeof(mesg4));
			sr_recv_iovec.add(mesg1, sizeof(mesg1));
			sr_recv_iovec.add(mesg2, sizeof(mesg2));
			sr_recv_iovec.add(mesg3, sizeof(mesg3));
					
			printf("testing linux >>>>>\n");

			//Read the data from client
			status = client[i]->readv(sr_recv_iovec, QUANTAnet_tcpClient_c::BLOCKING);
			
			//if status is OK
			if (status == QUANTAnet_tcpClient_c::OK)
			{
				//print the received data from the client
				printf("\nData received from Client# %d \n",i);	
				for(j = 0; j < 5; j++)				
					printf("mesg1: %d\n",mesg1[j]);				
				printf("\n");

				for(j = 0; j < 5; j++)				
					printf("mesg2: %f\n",mesg2[j]);				
				printf("\n");

				for(j = 0; j < 5; j++)				
					printf("mesg3: %f\n",mesg3[j]);				
				printf("\n");
						
				printf("mesg4: %s\n",mesg4);				
				printf("\n");

				//send text to the client
				sr_send_iovec.add(mesg5, sizeof(mesg5));
				sr_send_iovec.add(mesg6, sizeof(mesg6));

				status = client[i]->writev(sr_send_iovec, QUANTAnet_tcpClient_c::BLOCKING);
				
				//if status is OK
				if (status == QUANTAnet_tcpClient_c::OK) 
				{
					char name[256];
					aClient->getSelfIP(name);
					//Performance monitoring
					printf("Displaying Performance statistics for Client>> %d\n", i);
					client[i]->showStats("tcpServer_perf_monitoring", "Some_remoteClient_data");
					printf("\n");				
				}
				//if there is some error while writing
				else if (status != QUANTAnet_tcpClient_c::NON_BLOCKING_HAS_NO_DATA)
				{
					client[i]->showStatus(status, dataSize);						
				}
			}							
			//some error has occured while reading
			else if (status != QUANTAnet_tcpClient_c::NON_BLOCKING_HAS_NO_DATA)
			{
				printf("\nError reading from Client# %d\n",i);
				client[i]->showStatus(status, dataSize);										
			}//end of if

			//clear the iovec buffer to add new values
			sr_recv_iovec.removeAll();
			sr_send_iovec.removeAll();

		}//end of for

	}//end of while

    return 1;
}

