/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

/* This demonstrates the use of the QUANTAnet_tcpReflector_c class.
 * This server accepts N client connections.
 * It receives data from the clients and reflects it to all other connected clients.
 */

#include <QUANTA/QUANTA.hxx>

#include <stdlib.h>
#ifdef WIN32
#include <string.h>
#else
#include <strings.h>
#endif

#include <signal.h>

bool keepRunning = true;

void interruptHandler(int)
{
  printf("Ctrl-C caught...\n");
  keepRunning = false;
  signal(SIGINT, SIG_DFL);
}


int main (int argc, char *argv[])
{
  // Install signal handlers for catching the ctrl-c (SIGINT).
  signal(SIGINT, interruptHandler);

	QUANTAinit();

  QUANTAnet_tcpReflector_c* server = new QUANTAnet_tcpReflector_c;
	if (server->init() != QUANTAnet_tcpReflector_c::OK)	{
		return 1;
	}
    /*to create a logfile for storing performance monitoring results 
    */
    int fileOpenOK =0;
    FILE* filePtr;
    filePtr = fopen("TCPREFLECTOR.log", "a");
    if (filePtr)
    {        
        fileOpenOK =1;
    }

	void newclientcallback(QUANTAnet_tcpReflectorClient_c *newClient, void* userData);
	server->interceptNewConnection(newclientcallback, NULL);

	while (keepRunning) {
		server->checkForNewClients();
		server->process();
        //Uncoment this for an instantaneous display of performance monitoring statistics
        //	server->showStats("Remote_data_monitored","Some_remote_data");
        if (fileOpenOK)
                    {
                        if (server->logStats("CAVE_avatar_on_some_machine", "New_avatar_entered", filePtr) != QUANTAnet_perfMonitor_c::OK)
                            printf("\nlogStats Failed!\n");
                    }  
        QUANTAsleep(1);
	}

  // Close the file and delete the server.
  if (fileOpenOK)
    fclose(filePtr);
  delete server;
  return 0;
}

void newclientcallback(QUANTAnet_tcpReflectorClient_c *newClient, void* userData)
{
	char name[256];
	newClient->getRemoteIP(name);
	printf("NEW CLIENT: %s\n",name);

}
