/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

/* E.g. of TCP reflector client. This uses the QUANTAnet_tcpReflectorClient_c
 * class designed specifically for use with the QUANTAnet_tcpReflector_c
 * class.
 * This will connect to a server and send text to it. In order to be able to
 * see anything at all you need to have multiple clients connecting to a single
 * server.
 */
#include <QUANTA/QUANTA.hxx>

#include <stdlib.h>
#ifdef WIN32
#include <string.h>
#else
#include <strings.h>
#endif

/* This is the size of all messages sent from clients to servers. */
#define TRANSMITTED_DATA_SIZE 512

char sendBuffer[TRANSMITTED_DATA_SIZE];

int main (int argc, char *argv[])
{
	int tic = 0;
	int status, dataSize;

	QUANTAinit();
	QUANTAnet_tcpReflectorClient_c *client = new QUANTAnet_tcpReflectorClient_c;

	if (client->connectToServer(argv[1]) != QUANTAnet_tcpReflectorClient_c::OK)    {
		exit(1);
	}
    int perfDaemonStatus =1;
	if (argc > 2 && client->initSendStats(argv[2]) != QUANTAnet_perfMonitor_c::OK) {
		printf("Cannot connect to %s\n", argv[2]);
		delete client;
		exit(1);
	}
    if (argc <= 2) 
        perfDaemonStatus =0;

    /*to create a logfile for storing performance monitoring results 
      (logfile = "Machine_IP + Port")
     */
    char * fileName = new char[256];
    char IP[256];
    int port;
    int fileOpenOK =0;

    QUANTAnet_socketbase_c::getSelfIP(IP);
    port = client->getSelfPort();
  
    
    sprintf(fileName, "%s_%d.log", IP, port);
    
    FILE* filePtr;
    filePtr = fopen(fileName, "a");
    if (filePtr)
    {        
        fileOpenOK =1;
    }
    
	while(1) {
	  		char name[256];
		QUANTAnet_socketbase_c::getSelfIP(name);

		tic++;
		sprintf(sendBuffer,"%s TICK(%d)",name, tic);

		dataSize = TRANSMITTED_DATA_SIZE;
		printf("\n***Write***\n");
		status = client->write(sendBuffer, &dataSize);

		if (status != QUANTAnet_tcpReflectorClient_c::OK) {
		  		client->showStatus(status, dataSize);
		}

		dataSize = 0;
		char* receiveBuffer;

		printf("\n***Read***\n");
		status =  client->read(&receiveBuffer, &dataSize,
				 QUANTAnet_tcpReflectorClient_c::NON_BLOCKING);

		if (status == QUANTAnet_tcpReflectorClient_c::OK) {
			printf("Data received: %s\n", receiveBuffer);
	
            //Performance monitoring -ignore the first 10 packets
			if(tic >10)
				{
                    //Uncomment this for an instantaneous display locally
					//client->showStats("CAVE_avatar_on_clark", "New_avatar_entered");					
                    //if (perfDaemonStatus)
                    //{
                        //send stats to a perfdaemon
                        if (client->sendStats("CAVE_avatar_on_clark", "New_avatar_entered") != QUANTAnet_perfMonitor_c::OK)
                            printf("\nsendStats failed!\n");
                   // }
                    
                    //to log stats into a file
                    if (fileOpenOK)
                    {
                        if (client->logStats("CAVE_avatar_on_clark", "New_avatar_entered", filePtr) != QUANTAnet_perfMonitor_c::OK)
                            printf("\nlogStats Failed!\n");
                    }                    
				}
			delete receiveBuffer;

		} else if (status != QUANTAnet_tcpReflectorClient_c::NON_BLOCKING_HAS_NO_DATA)
			client->showStatus(status, dataSize);
        QUANTAsleep(1);
	}
    fclose(filePtr);
	client->exitSendStats();
	delete client;
        return 1;
}



