/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

/* 
 * UDP Server example. Server sets itself up and just listens for
 * incoming data.This class shows the use of QUANTAnet_iovec_c class 
 *
 * It receives QUANTAnet_iovec from the clients
 * It sends "server alive" message back to the clients 
 */

#include <QUANTA/QUANTA.hxx>
//extern int errno;

#ifdef WIN32
#include <string.h>
#else
#include <strings.h>
#include <sys/errno.h>
#endif

#define MAXLINE 256
#define DEBUG


/* See if the current client is the 1st time we've encountered it.
 * If it is then save it away.
 */
void CheckClient(QUANTAnet_udp_c *);


/* Array of pointers to clients that might connect with us. Allow 5 clients to connect. */
QUANTAnet_udp_c *bunchOfClients[5];
int numberOfClients = 0;


int main (int argc, char** argv)
{
	int    j;
	char   *selfIp = new char[256];
	int    status;

	//These variable are used to read data from QUANTAnet_iovec_c
	int    mesg1[5]; 
	float  mesg2[5]; 
	double mesg3[5]; 
	char   mesg4[6] = ""; mesg4[6]='\0';

	//variables used in sending data to clients
	char   mesg5[6] =""; mesg5[6]='\0';	
	char   mesg6[5] =""; mesg6[5]='\0';	
	
	sprintf(mesg5,"Server");
	sprintf(mesg6,"Alive");

	//create a QUANTAnet_iovec_c object which will contain 4 iovec structure
	//as you have 4 varaible to read
	//this object is used for receving data from cleints
	QUANTAnet_iovec_c sr_recv_iovec(4);

	//this object is used for sending data to clients
	QUANTAnet_iovec_c sr_send_iovec(2);


	QUANTAinit();

	QUANTAnet_udp_c *server;

	server = new QUANTAnet_udp_c;

	//Open a static port for others to connect to
	server->init(6000);

	while(1)
	{
		sr_recv_iovec.add(mesg4, sizeof(mesg4));
		sr_recv_iovec.add(mesg1, sizeof(mesg1));
		sr_recv_iovec.add(mesg2, sizeof(mesg2));
		sr_recv_iovec.add(mesg3, sizeof(mesg3));
		
		if ((status = server->readv(sr_recv_iovec, QUANTAnet_udp_c::BLOCKING)) == QUANTAnet_tcpClient_c::OK)
		{
			char ReceiveIP[256];
			server->getReceiveIP(ReceiveIP);
			printf("\nData received from Client IP:%s Port:%u\n",ReceiveIP,server->getReceivePort());
			
			for(j = 0; j < 5; j++)				
			printf("mesg1: %d\n",mesg1[j]);				
			printf("\n");

			for(j = 0; j < 5; j++)				
			printf("mesg2: %f\n",mesg2[j]);				
			printf("\n");

			for(j = 0; j < 5; j++)				
			printf("mesg3: %f\n",mesg3[j]);				
			printf("\n");
						
			printf("mesg4: %s\n",mesg4);				
			printf("\n");

			server->clone();
			server->copyReceiveAddressToSendAddress();
			
			//send text to the client
			sr_send_iovec.add(mesg5, sizeof(mesg5));
			sr_send_iovec.add(mesg6, sizeof(mesg6));

			status = server->writev(sr_send_iovec, QUANTAnet_udp_c::BLOCKING);

			// Get the destination IP address
			char name[256];
			server->getSendIP(name);

			//if status is OK
				if (status == QUANTAnet_udp_c::OK) 
				{
					printf("Sent data to Client. IP:%s Port:%u\n",name,server->getSendPort());
					
				}
				//if there is some error while writing
				else if (status != QUANTAnet_udp_c::NON_BLOCKING_HAS_NO_DATA)
				{
					//bunchOfClients[i]->showStatus(status, dataSize);						
					printf("\nError writing to Client. IP:%s Port:%u\n",name,server->getSendPort());
					printf("\nError no.:%s\n",status);
				}
		}
	    //some error has occured while reading
		else if (status != QUANTAnet_udp_c::NON_BLOCKING_HAS_NO_DATA)
		{
			char name[256];
			server->getReceiveIP();
			printf("\nError reading from Client. IP:%s Port:%u\n",name,server->getReceivePort());
													
		}//end of if

		//clear the iovec buffer to add new values
		sr_recv_iovec.removeAll();
		sr_send_iovec.removeAll();

		
	} /*end of while(1) */

    
} /* end of main */


// THIS DEMO PROGRAM DOES NOT USE THE BELOW CheckClient FUNCTION CALL

/* Check to see if the current client is one that we have not encountered
 * before.
 */
void CheckClient(QUANTAnet_udp_c *server)
{
	QUANTAnet_udp_c *eachClient;

	for (int i = 0; i < numberOfClients; i++) 
	{
		eachClient = bunchOfClients[i];
		if ((eachClient->getSendIP() == server->getReceiveIP()) &&
		    (eachClient->getSendPort() == server->getReceivePort())) 
			return;
	}
	printf("NEW CLIENT\n\n");

	bunchOfClients[numberOfClients] = server->clone();
	bunchOfClients[numberOfClients]->copyReceiveAddressToSendAddress();
	numberOfClients++;
}



