/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

/* UDP example to connect to a UDP server and send data to it. */

#include <QUANTA/QUANTA.hxx>

#ifdef WIN32
#include <string.h>
#else
#include <strings.h>
#endif
#include <stdlib.h>

#define MAXLINE 256

int count = 0;

int main(int argc, char** argv)
{
    int port;
	QUANTAinit();

	char recvline[MAXLINE];
	QUANTAnet_udp_c *mainUDPSocket = new QUANTAnet_udp_c;

	// Enable instrumentation
	mainUDPSocket->enableInstrumentation();

	/* Open a dynamic port since we are a client and don't need to
	 * commit to any one port.
	 */
	mainUDPSocket->init();
	
	// We recommend that you use the blocking mode option in 
	// the send() and recieve() instead of calling makeNonBlocking()
	// makeNonBlocking() will be slowly phased out.
	// The demo shows how you can use send() and receive() with blocking mode
	// Comment out the following line if you are using the blockingmode in send() and receive()
	mainUDPSocket->makeNonBlocking();

    if (argv[2])
        port = atoi(argv[2]);
    else port = 6000;        
	/* But we want to use the socket to send data to the server */
	mainUDPSocket->setSendAddress(argv[1],port);

	while(1) {

		char mesg[MAXLINE];
		int n;

		sprintf(mesg,"COUNT %d\n",count++);
		n = strlen(mesg) + 1;

		/* Send message to the server */
		if (mainUDPSocket->send(mesg, n) != n)
//		if (mainUDPSocket->send(mesg, n, QUANTAnet_udp_c::NON_BLOCKING) != n)
			printf("Client sendto error\n");

		/* To receive something in return from the server you can do something
		 * like:
		 */

		n = mainUDPSocket->receive(recvline, MAXLINE);
//		n = mainUDPSocket->receive(recvline, MAXLINE,QUANTAnet_udp_c::NON_BLOCKING);

		if (n < 0)  
			if (n == QUANTAnet_udp_c::NON_BLOCKING_HAS_NO_DATA)
				printf("Nonblocking has no data\n");
			else
				printf("Client receive error\n");
		else
		{
			printf("RECV SIZE %d ",n);
			printf(recvline);
		}
		// Remove this comment to see how performance monitoring is done.		
//		mainUDPSocket->showStats("UDP_Performance_monitoring", "Continuous_data_transfer_Client_Side");
		QUANTAsleep(10);

	}
        return 1;

}
