/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

/* UDP Server example. Server sets itself up and just listens for
 * incoming data. Then it prints it to the screen and echoes
 * a message back to all the clients that are connected to it.
 */

#include <QUANTA/QUANTA.hxx>
//extern int errno;

#ifdef WIN32
#include <string.h>
#else
#include <strings.h>
#include <sys/errno.h>
#endif

#define MAXSIZE 256


int main (int argc, char** argv)
{

	QUANTAinit();

	QUANTAnet_udp_c *mainUDPSocket;

	mainUDPSocket = new QUANTAnet_udp_c;

	// Enable instrumentation
	mainUDPSocket->enableInstrumentation();

	/* Open a static port for others to connect to. Hence you are a server */
	mainUDPSocket->init(6000);

	mainUDPSocket->makeNonBlocking();
	
	char recvline[MAXSIZE];

	while(1) {
		while (mainUDPSocket->receive(recvline, MAXSIZE,QUANTAnet_udp_c::BLOCKING) > 0) {

			/* Show who the client is by printing the IP address & port */
			printf("RECEIVED: %s",recvline);
			mainUDPSocket->printInfo();

			/* See if the current client is the 1st time we've encountered it.
			 * If it is then save it away.
			 */
			void CheckClient(QUANTAnet_udp_c *);
			CheckClient(mainUDPSocket);

			/* Send a reply to all the clients. */
			void ReplyToClients();
			ReplyToClients();
	// Remove the following comment and compile the demos again to see 
	// performance statistics		
//	            mainUDPSocket->showStats("UDP_Performance_monitoring", "Continuous_data_transfer_Server_Side");
		}
#ifdef WIN32
		if (errno != WSAEWOULDBLOCK) {
			printf("RECEIVE ERROR\n");
		}
#else
		if (errno != EWOULDBLOCK) {
			printf("RECEIVE ERROR\n");
		}
#endif
	}
        return 1;

}


/* Array of pointers to clients that might connect with us. Allow 5 clients to connect. */
QUANTAnet_udp_c *bunchOfClients[5];
int numberOfClients = 0;

/* Check to see if the current client is one that we have not encountered
 * before.
 */
void CheckClient(QUANTAnet_udp_c *mainUDPSocket)
{

	QUANTAnet_udp_c *eachClient;

	for (int i = 0; i < numberOfClients; i++) {
		eachClient = bunchOfClients[i];
		if ((eachClient->getSendIP() == mainUDPSocket->getReceiveIP()) &&
		    (eachClient->getSendPort() == mainUDPSocket->getReceivePort())) return;
	}
	printf("NEW CLIENT\n\n");
	bunchOfClients[numberOfClients] = mainUDPSocket->clone();
	bunchOfClients[numberOfClients]->copyReceiveAddressToSendAddress();
	numberOfClients++;
}

/* Reply to all clients that are conencted to me. */
void ReplyToClients()
{
	static int ok = 0;

	ok++;
	char mesg[256];
	for(int i = 0 ; i<numberOfClients; i++) {
		sprintf(mesg,"HOWDY CLIENT# %d\tTOTAL CLIENTS: %d MESSAGE: %d\n", i, numberOfClients, ok);
		bunchOfClients[i]->send(mesg, strlen(mesg)+1,QUANTAnet_udp_c::NON_BLOCKING);
	}

}

