/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include <QUANTA/QUANTA.hxx>
#include <signal.h>

volatile bool keepRunning = true;

void interruptHandler(int)
{
  printf("Ctrl-C caught...\n");
  keepRunning = false;
  signal(SIGINT, SIG_DFL);
}

int main(int argc, char** argv)
{
  if (argc < 2) {
    printf("usage: reflector <port> [perfserver]\n");
    return 1;
  }

  // Install signal handlers for catching the ctrl-c (SIGINT).
  signal(SIGINT, interruptHandler);

  QUANTAinit();

	QUANTAnet_udpReflector_c *reflector = new QUANTAnet_udpReflector_c;

	reflector->setIncomingPort(atoi(argv[1]));
	//	reflector->setForcedDestination(argv[2],atoi(argv[3]));
	//	reflector->setMulticastAddress(argv[4],atoi(argv[5]));
	reflector->init();
	//reflector->enableInstrumentation();

	double ct, lt;
	lt = 0;
    int perfDaemonStatus =1;
    if (argc > 2 && reflector->initSendStats(argv[2])
        != QUANTAnet_perfMonitor_c::OK)
        {
            printf("\nCannot connect to the perfDaemon\n");
            exit(1);
        }
    if (argc <= 2)
        perfDaemonStatus = 0;            //No perfDaemon IP given

    while (keepRunning) {
      // css: The UDP reflector creates its own thread for handling
      // connections. As a result, the process function is no longer
      // needed since it does nothing.
      // reflector->process();
      ct = reflector->getTimeInSecs();
      if ((ct - lt) > 5.0) {
        reflector->showStats("Monitoring_in_udpReflector",
                             "Testing_some_new_data");
        if (perfDaemonStatus)
        {
          if (reflector->sendStats("VIDEO_AVATAR_ON_CRAY",
                                   "UDP_TRACKER_DATA")
              !=QUANTAnet_perfMonitor_c::OK)
            printf("\nSendStats failed!\n");
        }
        lt = ct;
      }
      QUANTAsleep(1);
    }
    reflector->exitSendStats();
    delete reflector;
    return 0;
}
