/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

/*******************************************************************************
 * QUANTA Barrier Class Demo
 *
 * NOTE: adapted from "Thread Time The Multithreaded Programming Guide"
 *******************************************************************************/

#include <QUANTA/QUANTA.hxx>

QUANTAts_barrier_c *barrier;


void* threadfunc(void *data)
{
 int *id = ((int *)data);
 fprintf(stdout, "Thread %d going into barrier\n", id);
 if(barrier->wait() != 0)
 {
    fprintf(stderr, "thread barrier->wait() failed\n");
    exit(0);
 }
	
 fprintf(stdout, "Thread %d leaving barrier\n", id);
 return (void*) 0;
}


int main(int argc, char *argv[])
{
 QUANTAinit();
 QUANTAts_thread_c thread1, thread2;

 barrier = new QUANTAts_barrier_c(3);
 thread1.create(threadfunc, ((int *)1));
 QUANTAsleep(1);

 thread2.create(threadfunc, ((int *)2));
 QUANTAsleep(1);

 fprintf(stdout, "Main process going into barrier\n");
 if(barrier->wait() != 0)
 {
    fprintf(stderr, "main process barrier->wait() failed\n");
    exit(0);
 }
 
 fprintf(stdout, "Main process leaving barrier\n");
 QUANTAsleep(1);

 // halt main process so all threads can finish
 thread1.join();
 thread2.join();

 QUANTAexit();
 return 1;
}
