/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include <QUANTA/QUANTAdb_sharedState_c.hxx>
#include <QUANTA/QUANTAdb_c.hxx>
#include <string.h>

QUANTAdb_sharedState_c::QUANTAdb_sharedState_c()
{
	_path = new char[255];
	_path[0] = '\0';
	_key = new char[255];
	_key[0] = '\0';
	_dbclient = 0;
}

QUANTAdb_sharedState_c::QUANTAdb_sharedState_c( QUANTAdb_client_c *dbclient, char *path, char *key )
: _dbclient( dbclient ), _sharingEnabled( false )
{
	_path = new char[255];
	_key = new char[255];
	if (path)
		strcpy( _path, path );
	else
		path[0] = '\0';
	if (key)
		strcpy( _key, key );
	else
		fprintf(stderr,"ERROR! QUANTAdb_sharedState_c requires a valid key name.\n");
	if (_dbclient)
		_dbclient->attach( this );
}

QUANTAdb_sharedState_c::~QUANTAdb_sharedState_c()
{
	// do nothing since the pointer to the dbclient is external
	if (_dbclient)
		_dbclient->detach( this );
	delete[] _path;
	delete[] _key;
}

void QUANTAdb_sharedState_c::update( QUANTAmisc_subject_c *subj )
{
	QUANTAdb_client_c *theClient = (QUANTAdb_client_c*) subj;
	// check the path name
	char *path = theClient->getNotifyPathname();
	if (strcmp(_path, path) == 0)
	{
		// check the key name
		char *key = theClient->getNotifyKeyname();
		if (strcmp(_key, key) == 0)
		{
			// it's a match, so get the data
			int size;
			char *data = theClient->get( path, key, &size );
			this->unpackState( data, size );
			delete[] data;
		}
	}
}

int QUANTAdb_sharedState_c::refresh()
{
	if (_dbclient)
		return _dbclient->fetch( _path, _key );
	else
		return 0;
}
