/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include <QUANTA/QUANTAmisc_hash.hxx>

/* This hash function computes a 31 bit value.  The value is used to index
   the hash directory using the top n bits.  It is also used in a hash bucket
   to find the home position of the element by taking the value modulo the
   bucket hash table size. */
QUANTAmisc_hashWord_t QUANTAmisc_hash (char *key, int ksize){
	QUANTAmisc_hashWord_t value;	/* Used to compute the hash value.  */
	int   index;		/* Used to cycle through random values. */


	/* Set the initial value from key. */
	value = 0x238F13AF * ksize;
	for (index = 0; index < ksize; index++)
		value = (value + (key[index] << (index*5 % 24))) & 0x7FFFFFFF;
	
	value = (1103515243 * value + 12345) & 0x7FFFFFFF;  

	/* Return the value. */
	return((QUANTAmisc_hashWord_t) value);
}
