/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

/* File: QUANTAnet_fecServer_c.cxx
   Description: The implementation of QUANTAnet_fecServer_c class
*/

#include <iostream.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>

#include <QUANTAnet_fecServer_c.hxx>
//#define DEBUG
extern int errno;

QUANTAnet_fecServer_c::QUANTAnet_fecServer_c()
{
        srcUDPSocket = new QUANTAnet_udp_c;
        fecUDPSocket = new QUANTAnet_udp_c;
        fecEngine = new QUANTAnet_fecEngine_c();
}

QUANTAnet_fecServer_c::~QUANTAnet_fecServer_c()
{
	delete srcPackPool;
	delete fecPackPool;
	delete recvIndex;
	delete recoverReady;
	my2dDelete(recvBuff, _numRowofRecvBuff, _numTotalPack);
	myDelete(recvPack, _numSourcePack);
	for(int i=0; i<_numRowofRecvBuff; i++)
		delete recvFlag[i];
	delete recvFlag;
	delete srcUDPSocket;
	delete fecUDPSocket;
	fecEngine->release(fecMatrix);
	delete fecEngine;
		
}

// Set number of source pack in the whole block
void QUANTAnet_fecServer_c::setNumSourcePack(unsigned int numSourcePack)
{
	_numSourcePack = numSourcePack;
}

// Set number of packets in the whole block
void QUANTAnet_fecServer_c::setNumTotalPack(unsigned int numTotalPack)
{
	_numTotalPack = numTotalPack;
}

// Set the size of payload
void QUANTAnet_fecServer_c::setPayloadSize(unsigned int payloadSize)
{
	_payloadSize = payloadSize;
}

// Set number of rows of the receiving buffer
void QUANTAnet_fecServer_c::setNumRowofRecvBuff(unsigned short numRowofRecvBuff)
{
	_numRowofRecvBuff = numRowofRecvBuff;
}

// Set port number for QUANTAnet_udp_c object of source packet
int QUANTAnet_fecServer_c::setSrcReceivePort(unsigned short port)
{
	return srcUDPSocket->init(port);
}

// Set port number for QUANTAnet_udp_c object of FEC packet
int QUANTAnet_fecServer_c::setFecReceivePort(unsigned short port)
{
	return fecUDPSocket->init(port);
}

// Set timeout value
void QUANTAnet_fecServer_c::setTimeOut(unsigned short timeOut)
{
	_timeOut = timeOut;
}

// Get number of source pack in the whole block
unsigned int QUANTAnet_fecServer_c::getNumSourcePack()
{
	return _numSourcePack;
}

// Get number of packets in the whole block
unsigned int QUANTAnet_fecServer_c::getNumTotalPack()
{
	return _numTotalPack;
}

// Get the size of payload
unsigned int QUANTAnet_fecServer_c::getPayloadSize()
{
	return _payloadSize;
}

// Get number of rows of the receiving buffer
unsigned short QUANTAnet_fecServer_c::getNumRowofRecvBuff()
{
	return _numRowofRecvBuff;
}

// Get the timeout value. Timeout value is the max time for waiting a source packet coming.
unsigned short QUANTAnet_fecServer_c::getTimeOut()
{
	return _timeOut;
}

// Get QUANTAnet_udp_c pointer to the source packet socket 
QUANTAnet_udp_c* QUANTAnet_fecServer_c::getSrcUDPSocket()
{
	if(srcUDPSocket != NULL)
		return srcUDPSocket;
	else
		return NULL;
}

// Get QUANTAnet_udp_c pointer to the FEC packet socket
QUANTAnet_udp_c* QUANTAnet_fecServer_c::getFecUDPSocket()
{
	if(fecUDPSocket != NULL)
		return fecUDPSocket;
	else
		return NULL;
}

// Memory allocation/free functions
char*** QUANTAnet_fecServer_c::my2dNew(int x, int y, int size)
{
	char*** data;
	data = new char**[x];
	for(int i=0; i<x; i++)
		data[i] = new char*[y];
	for(int j=0; j<x; j++)
	  for(int k=0; k<y; k++)
	  	data[j][k] = new char[size];
	return data;
}

char** QUANTAnet_fecServer_c::myNew(int x, int size)
{
	char** data;
	data = new char*[x];
	for(int i=0; i<x; i++)
		data[i] = new char[size];
	return data;
}

void QUANTAnet_fecServer_c::my2dDelete(char*** data, int x, int y)
{
	for(int i=0; i<x; i++)
	  for(int j=0; j<y; j++)
	  	delete data[i][j];
	delete data;
}

void QUANTAnet_fecServer_c::myDelete(char** data, int x)
{
	for(int i=0; i<x; i++)
		delete data[i];
	delete data;
}

// Set RecvFlag for a packet. 0-Not Arrived, 1-Arrived, 2-Recovered
void QUANTAnet_fecServer_c::setRecvFlag(int x, int y, int value)
{
	recvFlag[x][y] = value;
}

// Clear RecvFlag
void QUANTAnet_fecServer_c::clearRecvFlag()
{
	for(unsigned int x=0; x<_numRowofRecvBuff; x++)
	  for(unsigned int y=0; y<_numTotalPack; y++)
	  	setRecvFlag(x,y,0);
}

// Get RecvFlag for a certain packet
unsigned short QUANTAnet_fecServer_c::getRecvFlag(int x, int y)
{
	return recvFlag[x][y];
}

// Check if the lost source packet can be recovered.
int QUANTAnet_fecServer_c::isRecoverReady()
{
	unsigned int num = 0;
	unsigned short flag;
	for(unsigned int i=0; i<_numTotalPack; i++)
	{
		flag = getRecvFlag(currentRowIndex,i);
		if( flag == 1 || flag == 2 )
			num++;
	}
	if(num >= _numSourcePack)
		return 1;
	else
		return 0;
}

// Construct a SourcePack from received message
void QUANTAnet_fecServer_c::packSrc(char* message)
{
	memcpy((void*)&srcPack.srcPackNum, (void*)message, sizeof(unsigned short));
	memcpy((void*)&srcPack.BlockNum, (void*)&message[sizeof(unsigned short)],
	sizeof(unsigned short));
	memcpy((void*)&srcPack.dataSize, (void*)&message[2*sizeof(unsigned
	short)], sizeof(unsigned int));
	srcPack.data = &message[2*sizeof(unsigned short)+sizeof(unsigned int)];
}

// Construct a FECPack from received message
void QUANTAnet_fecServer_c::packFec(char* message)
{
	memcpy((void*)&fecPack.fecPackNum, (void*)message, sizeof(unsigned short));
	memcpy((void*)&fecPack.BlockNum, (void*)&message[sizeof(unsigned short)],
	sizeof(unsigned short));
	memcpy((void*)&fecPack.dataSize, (void*)&message[2*sizeof(unsigned
	short)], sizeof(unsigned int));
	fecPack.data = &message[2*sizeof(unsigned short)+sizeof(unsigned int)];
}

// Calculation utilities
void QUANTAnet_fecServer_c::positionTranslate(unsigned short* x, unsigned short* y, unsigned short num)
{
	int position = num % _recvSrcBuffSize;
	*y = position % _numSourcePack;
	*x = (position-*y)/_numSourcePack;
}

// Store the received source packet in the receiving buffer with the right position
void QUANTAnet_fecServer_c::storeSrcPack(SourcePack* pack)
{
	int y = pack->srcPackNum;
	int x = pack->BlockNum % _numRowofRecvBuff;
	_countFecBlock = pack->BlockNum;
	_countSrcPack = pack->srcPackNum;
	// if run out of buffer, forward _countFinishedBlock
	if (_countFecBlock > _countFinishedBlock)
	{
	while (_countFecBlock > _countFinishedBlock + _numRowofRecvBuff)
	{
		_countFailedBlock ++;
		_countFailedPacket += countLostPackets();
		clearCurrentRowRecvFlag();
		_countFinishedBlock ++;
		currentRowIndex = (_countFinishedBlock + 1) % _numRowofRecvBuff;
//		fprintf(logFilePtr, "%d\tlost\n", _countFinishedBlock);
	}
	memcpy((void*)recvBuff[x][y], (void*)pack->data, pack->dataSize);
	setRecvFlag(x,y,1);
//	fprintf(logFilePtr, "src %d\t%d\n", _countFecBlock, _countSrcPack);
	}
}

// Store the received FEC packet in the receiving buffer with the right position
void QUANTAnet_fecServer_c::storeFecPack(FECPack* pack)
{
	int x = pack->BlockNum % _numRowofRecvBuff;
	int y = pack->fecPackNum + _numSourcePack;
	_countFecBlock = pack->BlockNum;
	_countFecPack = pack->fecPackNum;
	// if run out of buffer, forward _countFinishedBlock
	if (_countFecBlock > _countFinishedBlock)
	{
	while (_countFecBlock > _countFinishedBlock + _numRowofRecvBuff)
	{
		_countFailedBlock ++;
		_countFailedPacket += countLostPackets();
		clearCurrentRowRecvFlag();
		_countFinishedBlock ++;
		currentRowIndex = (_countFinishedBlock + 1) % _numRowofRecvBuff;
//		fprintf(logFilePtr, "%d\tlost\n", _countFinishedBlock);
	}
	memcpy((void*)recvBuff[x][y], (void*)pack->data, pack->dataSize);
	setRecvFlag(x,y,1);
//	fprintf(logFilePtr, "fec %d\t%d\n", _countFecBlock, _countFecPack);
	}
}

// Set a timer for timeout
void QUANTAnet_fecServer_c::setAlarm()
{
	gettimeofday(&startTime, (struct timezone *)NULL);
	msStartTime = startTime.tv_sec*1000 + startTime.tv_usec/1000.0;
	timeOutFlag = 1;
}

// Clear the timer
void QUANTAnet_fecServer_c::clearAlarm()
{
	msStartTime = 0.0;
	timeOutFlag = 0;
}

// Check if the timer is set
int QUANTAnet_fecServer_c::isAlarmEnabled()
{
	if(timeOutFlag == 1)
		return 1;
	else
		return 0;
}

// Check if it's timeout
int QUANTAnet_fecServer_c::isTimeOut()
{
	double period;
	struct timeval endTime;
	gettimeofday(&endTime, (struct timezone *)NULL);
	period = (endTime.tv_sec*1000 + endTime.tv_usec/1000) - msStartTime;
	if(period < _timeOut)
	{
		return 0;
	}
	else
	{
		clearAlarm();
		return 1;
	}
}

// Check how many packets are received and put them in the right order
void QUANTAnet_fecServer_c::calcRecvPack()
{
	unsigned int count = 0;
	for(unsigned int i=0; i<_numTotalPack; i++)
	{
		if(recvFlag[currentRowIndex][i] == 1 || recvFlag[currentRowIndex][i] == 2)
		{
			memcpy((void*)recvPack[count], (void*)recvBuff[currentRowIndex][i], _payloadSize);
			recvIndex[count++] = i;
			if(count >= _numSourcePack)
				break;
		}
	}
}

// Put the recovered source packets back into the receiving buffer
void QUANTAnet_fecServer_c::putBackRecoveredPack()
{
	for(unsigned int i=0; i<_numSourcePack; i++)
	{
		if(recvFlag[currentRowIndex][i] == 0)
		{
			memcpy((void*)recvBuff[currentRowIndex][i], (void*)recvPack[i], _payloadSize);
			setRecvFlag(currentRowIndex,i,2);
		}
	}
}

// Check if the expected packet is in the receiving buffer
int QUANTAnet_fecServer_c::isInRecvBuff(unsigned short seqNo)
{
	unsigned short x,y;
	positionTranslate(&x, &y, seqNo);
	int value = getRecvFlag(x,y);
	if(value == 1)
	{
		return PACKET_OK;
	}
	else if(value == 2)
	{
		return PACKET_RECOVERED;
	}
	else
	{
		return -1;
	}
}

// Calculation utilities
void QUANTAnet_fecServer_c::nextCurrentRowIndex()
{
	if(expectNextSrcPackNum % _numSourcePack == 0)
	{
		clearCurrentRowRecvFlag();
		currentRowIndex++;
		if(currentRowIndex >= _numRowofRecvBuff)
		{
			currentRowIndex = 0;
			//clearRecvFlag();
		}
	}
}

// Clear RecvFlag in the current row
void QUANTAnet_fecServer_c::clearCurrentRowRecvFlag()
{
	for(unsigned int y=0; y<_numTotalPack; y++)
		setRecvFlag(currentRowIndex,y,0);
}

void QUANTAnet_fecServer_c::nextExpectNextSrcPackNum()
{
	expectNextSrcPackNum++;
	if(expectNextSrcPackNum >= _maxBlockNum)
		expectNextSrcPackNum = 0;
	nextCurrentRowIndex();
}

void QUANTAnet_fecServer_c::calcRecvSrcBuffSize()
{
	_recvSrcBuffSize = _numSourcePack * _numRowofRecvBuff;
}

void QUANTAnet_fecServer_c::calcMaxBlockNum()
{
	int res = 65535 % _recvSrcBuffSize;
	_maxBlockNum = 65535 - res;
}

/* Initialize. The following parameters are needed: numSourcePack:number of source pack in the
	block, numTotalPack:number of packets in the block, payloadSize:size of payload,
	numRowofRecvBuff: number of rows of receiving buffer, timeout:timeout value,
	srcPort:source port number, fecPort:FEC port number */
void QUANTAnet_fecServer_c::init(unsigned int numSourcePack, unsigned int numTotalPack,
	unsigned int payloadSize, unsigned short numRowofRecvBuff, unsigned short timeOut, unsigned short srcPort, unsigned short fecPort)
{
	setNumSourcePack(numSourcePack);
	setNumTotalPack(numTotalPack);
	setPayloadSize(payloadSize);
	setNumRowofRecvBuff(numRowofRecvBuff);
	calcRecvSrcBuffSize();
	calcMaxBlockNum();
	setTimeOut(timeOut);
	clearAlarm();
	
	pickupX = 0;
	pickupY = 0;
	currentRowIndex = 0;
	expectNextSrcPackNum = 0;
	_countFinishedBlock = -1;
	_countSrcPack = 0;
	_countFecBlock = 0;
	_countRecoveredBlock = 0;
	_countFailedBlock = 0;
	_countFailedPacket = 0;
	
	offsite = 2*sizeof(unsigned short)+sizeof(unsigned int);
	len = offsite + _payloadSize;
	
	srcPackPool = new char[len];
	fecPackPool = new char[len];
	
	recvIndex = new int[_numSourcePack];
	recoverReady = new unsigned short[_numRowofRecvBuff];
	
	if((recvPack = myNew(_numSourcePack, _payloadSize)) == NULL)
		cerr << "<QUANTAnet_fecServer_c::init>allocating memery failed!" << endl;
		
	if((recvBuff = my2dNew(_numRowofRecvBuff, _numTotalPack, _payloadSize)) == NULL)
		cerr << "<QUANTAnet_fecServer_c::init>allocating memery failed!" << endl;
		
	recvFlag = new unsigned short*[_numRowofRecvBuff];
	for(int i=0; i<_numRowofRecvBuff; i++)
		recvFlag[i] = new unsigned short[_numTotalPack];
	clearRecvFlag();	
	
	srcUDPSocket->init(srcPort);
//	srcUDPSocket->makeNonBlocking();
	fecUDPSocket->init(fecPort);
//	fecUDPSocket->makeNonBlocking();
	
	fecMatrix = fecEngine->init(_numSourcePack, _numTotalPack);
	if(fecMatrix == NULL)
		cerr << "<QUANTAnet_fecServer_c::init>fecEngine init failed!" << endl;
	
}


void QUANTAnet_fecServer_c::streamingRecv(void(*exportFunc)(void*, int))
{
	int socketStatus[2], retval;
	QUANTAnet_socketbase_c * arrayOfSockets[2];
	struct timeval timeout;
	logFilePtr = fopen("//tmp//fec_recv.log", "w");

printf("log file /tmp/fec_recv.log opened, start receiving...\n");

	arrayOfSockets[0] = srcUDPSocket;
	arrayOfSockets[1] = fecUDPSocket;

	while (1)
	{
		timeout.tv_sec = _timeOut;
		timeout.tv_usec = 0;
		retval = QUANTAnet_socketbase_c :: selectSock(arrayOfSockets, socketStatus, 2, &timeout);

		/* time out */
		if (retval == 0)
		{
			#ifdef DEBUG
			printf("timeout, receiver quit!\n");
			#endif
			return;
		}
		else if (retval > 0)
		{
			/* received a source packet */
			if (socketStatus[0] == 1)
			{
				if(srcUDPSocket->receive(srcPackPool, len) == len)
				{
					packSrc(srcPackPool);
					storeSrcPack(&srcPack);
					packetRecvHandler(exportFunc);
				}
			}
			/* received a fec packet */
			else if (socketStatus[1] == 1)
			{
				if((fecUDPSocket->receive(fecPackPool, len)) == len)
				{
					packFec(fecPackPool);
					storeFecPack(&fecPack);
					packetRecvHandler(exportFunc);
				}
			}
		}
		/* select call failed */
		else 
		{
			cerr << "<QUANTAnet_fecServer_c::streamingrecv>fecServer select failed!" << endl;
		}
	}
	fclose(logFilePtr);
}
		
int QUANTAnet_fecServer_c::checkAllSrcDataRecv()
{
	unsigned short flag;
	for(unsigned int i=0; i<_numSourcePack; i++)
	{
		flag = getRecvFlag(currentRowIndex,i);
		if( flag != 1 && flag != 2 )
			return 0;
	}
	return 1;
}

int QUANTAnet_fecServer_c::countLostPackets()
{
	unsigned short count=0;
	for (unsigned int i=0; i<_numSourcePack; i++)
	{
		if (getRecvFlag(currentRowIndex, i) == 0)
			count ++;
	}
	return count;
}

void QUANTAnet_fecServer_c::packetRecvHandler(void(*exportFunc)(void*, int))
{
	#ifdef DEBUG
	fprintf(logFilePtr, "received src pack %d %d\n", _countFecBlock, _countSrcPack);
	#endif

	/* All source data packets for the expected block are received */
	while (checkAllSrcDataRecv() || isRecoverReady())
	{
		if (checkAllSrcDataRecv())
		{
			exportFunc(recvBuff[currentRowIndex][0], _numSourcePack * _payloadSize);
			clearCurrentRowRecvFlag();
			_countFinishedBlock ++;
			currentRowIndex = (_countFinishedBlock + 1) % _numRowofRecvBuff;

			struct timeval now;
			double recvTime;
			gettimeofday(&now, (struct timezone *)NULL);
			recvTime = (double)now.tv_usec*0.000001 + now.tv_sec; 
			fprintf(logFilePtr,"%d\t%f\n", _countFinishedBlock, recvTime);

			#ifdef DEBUG
			printf("received %d block\n", _countFinishedBlock);
			#endif
		}
		/* 		try to recover blocks and exportFunc and ++ */
		else
		{
			calcRecvPack();
			fecEngine->decode(fecMatrix, (void**)recvPack, recvIndex, _payloadSize, 
					_numSourcePack);
			putBackRecoveredPack();
			exportFunc(recvBuff[currentRowIndex][0], _numSourcePack * _payloadSize);
			clearCurrentRowRecvFlag();
			_countFinishedBlock ++;
			currentRowIndex = (_countFinishedBlock + 1) % _numRowofRecvBuff;
			_countRecoveredBlock ++;

			struct timeval now;
			double recvTime;
			gettimeofday(&now, (struct timezone *)NULL);
			recvTime = (double)now.tv_usec*0.000001 + now.tv_sec; 
			fprintf(logFilePtr, "%d\t%f\trs\n", _countFinishedBlock, recvTime);

			#ifdef DEBUG
			printf("recovered %d block\n", _countFinishedBlock);
			#endif
		}
	}
}
		

void QUANTAnet_fecServer_c::printStatisticsData()
{
	double lostRate;
	lostRate = (double) _countFailedPacket / ((double) (_countFinishedBlock * _numSourcePack));
	printf("Total Blocks: %d\n", _countFinishedBlock);
	printf("Recovered Blocks: %d\n", _countRecoveredBlock);
	printf("Failed Blocks: %d\n", _countFailedBlock);
	printf("Failed Packets: %d\n", _countFailedPacket);
	printf("Total Data: %d Bytes\n", _countFinishedBlock * _payloadSize * _numSourcePack);
	printf("Loss Rate: %f\n", lostRate);
}



