/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include <QUANTA/QUANTAnet_objectStreamBase_c.hxx>


QUANTAnet_objectStreamBase_c::QUANTAnet_objectStreamBase_c(char *host, int port)
{
	_send_buffer = NULL;
	_receive_buffer = NULL;

	if(host)
		_host = strdup(host);
//	else if(getenv("QUANTA_STREAM_HOST"))
//		_host = strdup(getenv("QUANTA_STREAM_HOST"));
	else
//		_host = "localhost";
		_host = "127.0.0.1";


	if(port > 0)
		_port = port;
//	else if(getenv("QUANTA_STREAM_PORT"))
//		_port = atoi(getenv("QUANTA_STREAM_PORT"));
	else
		_port = 9575; // default port

}


QUANTAnet_objectStreamBase_c::~QUANTAnet_objectStreamBase_c(void)
{
	delete [] _host;
	delete [] _send_buffer;
	delete [] _receive_buffer;

}


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


QUANTAnet_objectStreamBaseTcp_c::QUANTAnet_objectStreamBaseTcp_c(char *host, int port)
	: QUANTAnet_objectStreamBase_c(host, port)
{
	_client = new QUANTAnet_tcpClient_c;
	if(_client->connectToServer(_host, _port) < 0)
	{
		printf("QUANTAnet_objectStreamBaseTcp_c() ERROR: Failed to connect to host: %s,  port: %i\n", _host, _port);
		exit(EXIT_FAILURE);
	}
}


QUANTAnet_objectStreamBaseTcp_c::QUANTAnet_objectStreamBaseTcp_c(void)
	: QUANTAnet_objectStreamBase_c(NULL, 0)
{
	_client = NULL;
}


QUANTAnet_objectStreamBaseTcp_c::~QUANTAnet_objectStreamBaseTcp_c(void)
{
	delete _client;
}


QUANTAnet_tcpClient_c *QUANTAnet_objectStreamBaseTcp_c::getNetworkClient(void)
{
	if(_client == NULL)
		return NULL;
	else
		return _client;
}


// assumes client is already connected to a tcp server
void QUANTAnet_objectStreamBaseTcp_c::setNetworkClient(QUANTAnet_tcpClient_c *client)
{
	if(client != NULL)
		_client = client;
}


int QUANTAnet_objectStreamBaseTcp_c::readObject(QUANTAnet_serializable_c *obj)
{
	int data_size = obj->getDataSize();
	if(data_size <= 0) // throw exception
	{
		fprintf(stderr, "QUANTAnet_objectStreamBaseTcp_c::readObject() - object data size is <= 0\n");
		exit(EXIT_FAILURE);
	}

	// init a new buffer	
	_receive_buffer = new char[data_size];
	// read the buffer from the network
	if(_client->read(_receive_buffer, &data_size, QUANTAnet_tcpClient_c::BLOCKING) > 0)
	{
		// read the serializable object's custom contents from the buffer using the datapacker functions
		obj->readObjectStream(_receive_buffer);
		delete [] _receive_buffer;
		return 1;
	}

	delete [] _receive_buffer;
	return 0;
}


int QUANTAnet_objectStreamBaseTcp_c::writeObject(QUANTAnet_serializable_c *obj)
{
	int data_size = obj->getDataSize();
	if(data_size <= 0) // throw exception
	{
		fprintf(stderr, "QUANTAnet_objectStreamBaseTcp_c::writeObject() - object data size is <= 0\n");
		exit(EXIT_FAILURE);
	}

	// init a new buffer
	_send_buffer = new char[data_size];
	// write the serializable object's custom contents to the buffer using the datapacker functions
	obj->writeObjectStream(_send_buffer);
	// send the buffer over the network
	if(_client->write(_send_buffer, &data_size, QUANTAnet_tcpClient_c::BLOCKING) > 0)
	{
		delete [] _send_buffer;
		return 1;
	}

	delete [] _send_buffer;
	return 0;
}


void QUANTAnet_objectStreamBaseTcp_c::close(void)
{
	_client->close();
}



////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


QUANTAnet_objectStreamBaseUdp_c::QUANTAnet_objectStreamBaseUdp_c(char *host, int port)
	: QUANTAnet_objectStreamBase_c(host, port)
{
	_client = new QUANTAnet_udp_c();
	_client->init();
	if(_client->setSendAddress(_host, _port) == 0)
	{
		printf("QUANTAnet_objectStreamBaseUdp_c() : ERROR - Failed to connect to host: %s,  port: %i\n", _host, _port);
		exit(EXIT_FAILURE);
	}
}


QUANTAnet_objectStreamBaseUdp_c::QUANTAnet_objectStreamBaseUdp_c(void)
	: QUANTAnet_objectStreamBase_c(NULL, 0)
{
	_client = NULL;
}


QUANTAnet_objectStreamBaseUdp_c::~QUANTAnet_objectStreamBaseUdp_c(void)
{
	delete _client;
}


QUANTAnet_udp_c *QUANTAnet_objectStreamBaseUdp_c::getNetworkClient(void)
{
	if(_client == NULL)
		return NULL;
	else
		return _client;
}


// assumes client is already connected to a udp server
void QUANTAnet_objectStreamBaseUdp_c::setNetworkClient(QUANTAnet_udp_c *client)
{
	if(client != NULL)
		_client = client;
}


int QUANTAnet_objectStreamBaseUdp_c::readObject(QUANTAnet_serializable_c *obj)
{
	int data_size = obj->getDataSize();
	if(data_size <= 0) // throw exception
	{
		fprintf(stderr, "QUANTAnet_objectStreamBaseUdp_c::readObject() - object data size is <= 0\n");
		exit(EXIT_FAILURE);
	}

	// init a new buffer	
	_receive_buffer = new char[data_size];
	// read the buffer from the network
	if(_client->receive(_receive_buffer, data_size, QUANTAnet_udp_c::BLOCKING) > 0)
	{
		// read the serializable object's custom contents from the buffer using the datapacker functions
		obj->readObjectStream(_receive_buffer);
		delete [] _receive_buffer;
		return 1;
	}
	

	delete [] _receive_buffer;
	return 0;
}


int QUANTAnet_objectStreamBaseUdp_c::writeObject(QUANTAnet_serializable_c *obj)
{
	int data_size = obj->getDataSize();
	if(data_size <= 0) // throw exception
	{
		fprintf(stderr, "QUANTAnet_objectStreamBaseUdp_c::writeObject() - object data size is <= 0\n");
		exit(EXIT_FAILURE);
	}

	// init a new buffer
	_send_buffer = new char[data_size];
	// write the serializable object's custom contents to the buffer using the datapacker functions
	obj->writeObjectStream(_send_buffer);
	// send the buffer over the network
	if(_client->send(_send_buffer, data_size, QUANTAnet_udp_c::BLOCKING) > 0)
	{
		delete [] _send_buffer;
		return 1;
	}

	delete [] _send_buffer;
	return 0;
}


void QUANTAnet_objectStreamBaseUdp_c::close(void)
{
	_client->close();
}
