/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include <QUANTAnet_rbudpBase_c.hxx>

void QUANTAnet_rbudpBase_c::passiveUDP(char *host)
{
	struct sockaddr_in cliaddr;
	struct hostent *phe;

	// Create a UDP sink 
	if ((udpSockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
	{
	        perror("socket error");
        	exit(1);
	}
	
	bzero((char *)&udpServerAddr, sizeof(udpServerAddr));
	udpServerAddr.sin_family = AF_INET;
	udpServerAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	udpServerAddr.sin_port = htons(udpLocalPort);

	if((bind(udpSockfd, (struct sockaddr *)&udpServerAddr, sizeof(udpServerAddr))) < 0) {
	        perror("UDP bind error");
	        exit(1);
	}

       // Use connected UDP to receive only from a specific host and port.
        bzero(&cliaddr, sizeof(cliaddr));
 	if (phe = gethostbyname(host))
		memcpy(&cliaddr.sin_addr, phe->h_addr, phe->h_length);
	else if ((cliaddr.sin_addr.s_addr = inet_addr(host)) == INADDR_NONE)
	{
		perror("can't get host entry");
		exit(1);
	}

        cliaddr.sin_family = AF_INET;
        cliaddr.sin_port = htons(udpRemotePort);
        if (connect(udpSockfd, (struct sockaddr *) &cliaddr, sizeof(cliaddr)) < 0)
        {
                perror("connect() error");
                exit(1);
        }
}

int QUANTAnet_rbudpBase_c::connectTCP(char * host)
{
	static struct sockaddr_in tcpServerAddr;
	int retval = 0;
	struct hostent *phe;
	struct timeval start, now;

	/*Create a TCP connection */
	bzero((char *)&tcpServerAddr, sizeof(tcpServerAddr));
	tcpServerAddr.sin_family = AF_INET;
	if (phe = gethostbyname(host))
		memcpy(&tcpServerAddr.sin_addr, phe->h_addr, phe->h_length);
	else if ((tcpServerAddr.sin_addr.s_addr = inet_addr(host)) == INADDR_NONE)
	{
		perror("can't get host entry");
		exit(1);
	}
	tcpServerAddr.sin_port = htons(tcpPort);
	
	if ((tcpSockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
	        perror("socket error");
	        exit(1);
	}

	gettimeofday(&start, NULL);
	do {
		retval = connect(tcpSockfd, (struct sockaddr *)&tcpServerAddr, sizeof(tcpServerAddr));
		gettimeofday(&now, NULL);
	}
	while ((retval < 0) &&  (USEC(&start, &now) < 5000000));
	return retval;
}

void QUANTAnet_rbudpBase_c::connectUDP(char *host)
{
	static struct sockaddr_in udpClientAddr;
	struct hostent *phe;

	// Fill in the structure whith the address of the server that we want to send to
	// udpServerAddr is class global variable, will be used to send data
	bzero(&udpServerAddr, sizeof(udpServerAddr));
	udpServerAddr.sin_family = AF_INET;
	if (phe = gethostbyname(host))
		memcpy(&udpServerAddr.sin_addr, phe->h_addr, phe->h_length);
	else if ((udpServerAddr.sin_addr.s_addr = inet_addr(host)) == INADDR_NONE)
	{	
		perror("can't get host entry");
		exit(1);
	}
	udpServerAddr.sin_port = htons(udpRemotePort);

	/* Open a UDP socket */
	if ((udpSockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
	{
       		perror("socket error");
        	exit(1);
	}

	// Allow the port to be reused.
	int yes = 1;
	if (setsockopt(udpSockfd,SOL_SOCKET,SO_REUSEADDR,&yes,sizeof(int)) == -1) {
        	perror("setsockopt");
        	exit(1);
	} 

	/* Bind any local address for us */
	bzero(&udpClientAddr, sizeof(udpClientAddr));
	udpClientAddr.sin_family = AF_INET;
	udpClientAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	udpClientAddr.sin_port = htons(udpLocalPort);
	if((bind(udpSockfd, (struct sockaddr *)&udpClientAddr, sizeof(udpClientAddr))) < 0) {
       		perror("UDP client bind error");
        	exit(1);
	}
}

void QUANTAnet_rbudpBase_c::initTCPServer()
{
	struct sockaddr_in tcpServerAddr;

	// Create TCP connection as a server in order to transmit the error list 
	// Open a TCP socket 
	if((listenfd = socket(AF_INET, SOCK_STREAM, 0))<0) {
	        perror("socket error");
	        exit(1);
	}

	int yes=1;
	if (setsockopt(listenfd,SOL_SOCKET,SO_REUSEADDR,&yes,sizeof(int)) == -1) {
        	perror("setsockopt");
        	exit(1);
	} 

	/* Bind our local address so that the client can send to us */
	bzero(&tcpServerAddr, sizeof(tcpServerAddr));
	tcpServerAddr.sin_family = AF_INET;
	tcpServerAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	tcpServerAddr.sin_port = htons(tcpPort);

	if((bind(listenfd, (struct sockaddr *)&tcpServerAddr, sizeof(tcpServerAddr))) < 0) {
	        perror("bind error");
	        exit(1);
	}
	
	if((listen(listenfd, 5))<0) {
	        perror("listen error");
	        exit(1);
	}
}

void QUANTAnet_rbudpBase_c::listenTCPServer()
{
  struct sockaddr_in tcpClientAddr;
	socklen_t clilen;

	clilen = sizeof(tcpClientAddr);
	tcpSockfd = accept(listenfd, (struct sockaddr *) &tcpClientAddr, &clilen);

	if (tcpSockfd < 0)
	{
	        perror("accept error");
	        exit(1);
	}
}

int QUANTAnet_rbudpBase_c::readn(register int fd, register char *ptr, register int nbytes)
{
        int nleft, nread;
        nleft = nbytes;
        while(nleft > 0)
        {
                nread = read(fd, ptr, nleft);
                if (nread < 0)
                        return(nread); /*error */
                else if(nread == 0)
                        break;  /* EOF */

                nleft -= nread;
                ptr += nread;
        }
        return (nbytes - nleft);
}

int QUANTAnet_rbudpBase_c::writen(register int fd, register char *ptr, register int nbytes)
{
int nleft, nwritten;

nleft = nbytes;
while(nleft > 0)
{
        nwritten = write(fd,ptr, nleft);
        if (nwritten <= 0)
                return (nwritten); /* error */
        nleft -= nwritten;
        ptr += nwritten;
}
return (nbytes -nleft);
}

int QUANTAnet_rbudpBase_c::reportTime(struct timeval & start)
{
        struct timeval end;
        int usecs;
        gettimeofday(&end, NULL);
        usecs = 1000000 * (end.tv_sec - start.tv_sec) + (end.tv_usec - start.tv_usec);
        start.tv_sec = end.tv_sec;
        start.tv_usec = end.tv_usec;
	return usecs;
}



void QUANTAnet_rbudpBase_c::initErrorBitmap()
{
	int i;
	// the first byte is 0 if there is error.  1 if all done.
	int startOfLastByte = totalNumberOfPackets - (sizeofErrorBitmap-2)*8;
	char bits[8] = {0x0001, 0x0002, 0x0004, 0x0008, 0x0010, 0x0020, 0x0040, 0x0080};

	/* The first byte is for judging all_done */
	for (i=0;i<sizeofErrorBitmap;i++)
	        errorBitmap[i] = 0;
	/* Preset those bits unused */
	for (i=startOfLastByte;i<8;i++)
        	errorBitmap[sizeofErrorBitmap-1] |= bits[i];
}

void QUANTAnet_rbudpBase_c::updateErrorBitmap(long seq)
{
	long index_in_list, offset_in_index;
	char bits[8] = {0x0001, 0x0002, 0x0004, 0x0008, 0x0010, 0x0020, 0x0040, 0x0080};
	// seq starts with 0
	index_in_list = seq >> 3;
	index_in_list ++;
	offset_in_index = seq%8;
	errorBitmap[index_in_list] |= bits[offset_in_index];
}


// return the count of errors
// The first byte is reserved to indicate if any packet's missing
int QUANTAnet_rbudpBase_c::updateHashTable()
{
int count = 0;
int i,j;
char bits[8] = {0x0001, 0x0002, 0x0004, 0x0008, 0x0010, 0x0020, 0x0040, 0x0080};

for (i=1;i<sizeofErrorBitmap; i++)
{
        for (j=0;j<8;j++)
        {
                if ((errorBitmap[i] & bits[j]) == 0)
                {
                        hashTable[count] = (i-1)*8+j;
                        count ++;
                }
        }
}
// set the first byte to let the sender know "all done"
if (count == 0)
        errorBitmap[0] = 1;
return count;
}


