/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include <QUANTA/QUANTAts_barrier_c.hxx>


QUANTAts_barrier_c::QUANTAts_barrier_c(int num_threads)
{
 _barrier_val = num_threads;
 _blocked_threads = 0;
 _predicate = 0;
}


QUANTAts_barrier_c::~QUANTAts_barrier_c(void)
{
}


int QUANTAts_barrier_c::wait(void)
{
 int ret_val, predicate;
 
 _mutex.lock();
 predicate = _predicate;
 
 /* increment blocked thread counter and perform barrier operation */
 _blocked_threads++;
 if(_blocked_threads == _barrier_val)
 {
    /* reset barrier for its next use */
    _predicate += 1;
    _blocked_threads = 0;
    
    /* last thread: wake up all other blocked threads */
    ret_val = _cv.broadcastSignal();
    if(ret_val != 0)
    {
       _mutex.unlock();
       return ret_val;
    }
 }
 
 else
 {
    /* wait until all threads reach this point */
    while(_predicate == predicate)
    {
       ret_val = _cv.wait(&_mutex);
       if(ret_val != 0)
       {
          _mutex.unlock();
          return ret_val;
       }
    }
 }
 
 _mutex.unlock();


 return 0;
}
