/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include <QUANTA/QUANTAts_rwlock_c.hxx>
#include <QUANTA/QUANTAmisc_debug.hxx>
#include <string.h>

inline void
showDebugMessage(const short& debugStatusOn, const char* debugMessage,
		 const char* infoMessage)
{
  if (debugStatusOn) {
    if (strlen(debugMessage) > 0) {
      cvrnMesg(const_cast<char*>(debugMessage), const_cast<char*>(infoMessage));
    }
  }
}

QUANTAts_rwlock_c::QUANTAts_rwlock_c()
  : debugStatusOn(0)
{
  debugMessage[0] = '\0';
#ifdef QUANTA_USE_PTHREADS
  int result = pthread_rwlock_init(&this->rwlock, NULL);
  if (result != 0) {
  }
#elif defined(QUANTA_USE_WINDOWSTHREADS)
#error Windows rwlock support has not been implemented!
#endif /* QUANTA_USE_WINDOWS_THREADS */
}

QUANTAts_rwlock_c::~QUANTAts_rwlock_c()
{
#ifdef QUANTA_USE_PTHREADS
  int result = pthread_rwlock_destroy(&this->rwlock);
  if (result != 0) {
  }
#elif defined(QUANTA_USE_WINDOWSTHREADS)
#error Windows rwlock support has not been implemented!
#endif /* QUANTA_USE_WINDOWS_THREADS */
}

int
QUANTAts_rwlock_c::readLock()
{
  showDebugMessage(this->debugStatusOn, this->debugMessage,
		   "BEFORE READ LOCK");
#ifdef QUANTA_USE_PTHREADS
  int result = pthread_rwlock_rdlock(&this->rwlock);
  if (result != 0) {
  }
#elif defined(QUANTA_USE_WINDOWSTHREADS)
#error Windows rwlock support has not been implemented!
#endif /* QUANTA_USE_WINDOWS_THREADS */
  showDebugMessage(this->debugStatusOn, this->debugMessage,
		   "AFTER READ LOCK");
  return result;
}

int
QUANTAts_rwlock_c::writeLock()
{
  showDebugMessage(this->debugStatusOn, this->debugMessage,
		   "BEFORE WRITE LOCK");
#ifdef QUANTA_USE_PTHREADS
  int result = pthread_rwlock_wrlock(&this->rwlock);
  if (result != 0) {
  }
#elif defined(QUANTA_USE_WINDOWSTHREADS)
#error Windows rwlock support has not been implemented!
#endif /* QUANTA_USE_WINDOWS_THREADS */
  showDebugMessage(this->debugStatusOn, this->debugMessage,
		   "AFTER WRITE LOCK");
  return result;  
}

int
QUANTAts_rwlock_c::tryReadLock()
{
#ifdef QUANTA_USE_PTHREADS
  int result = pthread_rwlock_tryrdlock(&this->rwlock);
  if (result != 0) {
  }
#elif defined(QUANTA_USE_WINDOWSTHREADS)
#error Windows rwlock support has not been implemented!
#endif /* QUANTA_USE_WINDOWS_THREADS */
  return result;
}

int
QUANTAts_rwlock_c::tryWriteLock()
{
#ifdef QUANTA_USE_PTHREADS
  int result = pthread_rwlock_trywrlock(&this->rwlock);
  if (result != 0) {
  }
#elif defined(QUANTA_USE_WINDOWSTHREADS)
#error Windows rwlock support has not been implemented!
#endif /* QUANTA_USE_WINDOWS_THREADS */
  return result;
}

int
QUANTAts_rwlock_c::unlock()
{
  showDebugMessage(this->debugStatusOn, this->debugMessage,
		   "BEFORE UNLOCK");
#ifdef QUANTA_USE_PTHREADS
  int result = pthread_rwlock_unlock(&this->rwlock);
  if (result != 0) {
  }
#elif defined(QUANTA_USE_WINDOWSTHREADS)
#error Windows rwlock support has not been implemented!
#endif /* QUANTA_USE_WINDOWS_THREADS */
  showDebugMessage(this->debugStatusOn, this->debugMessage,
		   "AFTER UNLOCK");
  return result;
}

void
QUANTAts_rwlock_c::showDebug(const short& debugStatus)
{
  this->debugStatusOn = debugStatus ? 1 : 0;
}

void
QUANTAts_rwlock_c::setDebugMessage(const char* msg)
{
  if (msg != NULL) {
    strncpy(this->debugMessage, msg, 1024);
  }
}

QUANTA_RWLOCK_T*
QUANTAts_rwlock_c::getRWLock()
{
  return &this->rwlock;
}
