/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include <QUANTA/md5Key_c.hxx>

#include <stdio.h>
#include <string.h>

md5Key_c::md5Key_c()
{
  memset(key, 0, MD5KEY_SIZE);
}

int md5Key_c::operator=(md5Key_c& copy)
{
  memcpy(keystr, copy.keystr, MD5KEY_SIZE_2);
  memcpy(key, copy.key, MD5KEY_SIZE);
  return *this;
}

int md5Key_c::hash(char *str, int len)
{
  if (len < 1)
    return 1;

  //unsigned char *buf = (unsigned char*) str;

  unsigned char *buf = new unsigned char[len];

  if (!buf) return 0;

  MD5_CTX ctx;
  
  memcpy(buf,str,len);

  MD5Init(&ctx);
  MD5Update(&ctx, buf, len);
  MD5Final(key, &ctx);

  delete buf;

  return 1;
}

md5Key_c::operator int()
{
  // Basically it extracts the first 4 bytes and makes it an int.
  int *front = (int *) key;
  int copy;
  memcpy((char*) &copy, front, sizeof(int));
  if (copy < 0) copy = -copy;
  return copy;
}

int md5Key_c::operator==(md5Key_c &nkey)
{
  if (memcmp(nkey.key,key,MD5KEY_SIZE) ==0)
    return 1;
  else
    return 0;
}


int md5Key_c::operator=(unsigned char* copy)
{
  for (int i =0; i < MD5KEY_SIZE; i++)
    key[i] = copy[i];

  return *this;
}

void md5Key_c::display()
{
  printf(getKeyAsStr());
  printf("\n");
}

/// Retrieve MD5 key as a string that is printable.
char* md5Key_c::getKeyAsStr()
{
  int i;
  char twoByte[3];

  memset(keystr, 0, 33);

  unsigned char* p = key;
  for (i = 0; i < MD5KEY_SIZE; ++i) {
    sprintf(twoByte,"%02x", *p++);
    strcat(keystr,twoByte);
  }

  return keystr;
}
