/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include <QUANTA/QUANTAnet_datapack_c.hxx>
#include <QUANTA/QUANTAnet_udp_c.hxx>

#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#ifdef WIN32
#ifndef __HAVE_WINSOCK2_H
#include <winsock2.h>
#define __HAVE_WINSOCK2_H
#endif /* __HAVE_WINSOCK2_H */
#include <sys\timeb.h>
#else
#include <sys/timeb.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/uio.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <sys/errno.h>
#include <signal.h>
#include <netdb.h>
#include <sys/param.h>
#include <sys/time.h>
#endif

class QUANTAnet_udp_c;

int _quanta_getSizeOfPerformanceMonitoringHeader() {
    QUANTAnet_datapack_c packer;
	return packer.sizeof_double();
}

void _quanta_prependPerformanceMonitoringHeader(char* newmsg, char* message, int length)
{
    //used to pack the timestamp with the message to be transmitted across the network
    QUANTAnet_datapack_c packer;
    
    //used just to find the time in Seconds
    QUANTAnet_udp_c dummyClient;

    int packingLength = length +  packer.sizeof_double();
    double timeStamp = dummyClient.getTimeInSecs();
    
    packer.initPack(newmsg, packingLength);
    packer.packDouble(timeStamp);
    packer.pack(message, length);
}

char* _quanta_allocMemForPerformanceMonitoredMesg(int length, int * newlength)
{
    QUANTAnet_datapack_c packer;
	*newlength = length + packer.sizeof_double();
	char* newmsg = new char[*newlength];
	return newmsg;
}

	
void _quanta_unprependPerformanceMonitoringHeader(double *remoteTimeStamp, double *inlatency, char* newmsg, int newlen,  char* umessage, int *ulen)
{
	if (newlen <=0) return;

    //used just to find the time in Seconds
    QUANTAnet_udp_c dummyClient;

    //used to unpack the timestamp from the buffer read
    QUANTAnet_datapack_c packer;

    int realSize = newlen - packer.sizeof_double();
    double localTimeStamp = dummyClient.getTimeInSecs();
    packer.initUnpack(newmsg,newlen);
    packer.unpackDouble(remoteTimeStamp);
    packer.unpack(umessage, realSize);    
	*inlatency = fabs(localTimeStamp - *remoteTimeStamp) ;
	*ulen = realSize;
}

