#ifdef _WIN32_WCE
#include <windows.h>
#include <string.h>
#include "sys\types.h"
#include "sys\stat.h"
#include "unicodecvt.h"
#include "sys\timeb.h"

int stat(const char *fileName, struct stat *buffer)
{
	int length;
	char* ext;
	HANDLE hFile;
	BY_HANDLE_FILE_INFORMATION fileInformation;
	LPWSTR lpwFileName = AnsiToUnicode((char*) fileName, &length);

	if (lpwFileName == NULL) return -1;

    hFile = CreateFile(lpwFileName, 0, FILE_SHARE_READ, NULL, 
	                   OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	free(lpwFileName);
	if (hFile == INVALID_HANDLE_VALUE) {
		hFile = 0;
		return -1;
	}
	GetFileInformationByHandle(hFile, &fileInformation);
	CloseHandle(hFile);

	buffer->st_gid = 0;    // no meaning on Windows CE device
	buffer->st_uid = 0;    // no meaning on Windows CE device
	buffer->st_ino = 0;    // no meaning on Windows CE device
	buffer->st_nlink = 1;  // no meaning on Windows CE device

	// This usually represents a value of the drive name such as 'C', but has
	// no meaning on Windows CE device. Thus, I am just reporting the 
	// unique number of the device where the file is on.
	buffer->st_rdev = fileInformation.dwVolumeSerialNumber;
	buffer->st_dev = fileInformation.dwVolumeSerialNumber;

	// Since we are only dealing with 32 bit file system with Windows CE, 
	// we don't really need dwFileSizeHigh field.
	buffer->st_size = fileInformation.nFileSizeLow; 

	// st_atime or st_ctime might contain 0 if underlying file system do not 
	// maintain the last access time or creation time.
	buffer->st_atime = GetFileTimeAfter1970(&(fileInformation.ftLastAccessTime));
	buffer->st_ctime = GetFileTimeAfter1970(&(fileInformation.ftCreationTime));

	buffer->st_mtime = GetFileTimeAfter1970(&(fileInformation.ftLastWriteTime));

	buffer->st_mode = 0;
	if (fileInformation.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) buffer->st_mode |= _S_IFDIR;
	else buffer->st_mode |= _S_IFREG;
	if (fileInformation.dwFileAttributes & FILE_ATTRIBUTE_READONLY) buffer->st_mode |= _S_IREAD;
	else {
		buffer->st_mode |= _S_IREAD;
		buffer->st_mode |= _S_IWRITE;
	}
	ext = strrchr(fileName, '.');
	if (ext) {
		char* tempExt = ext + 1;
		
		if (tempExt && *tempExt && _stricmp(tempExt, "EXE") == 0) { // executable
			buffer->st_mode |= _S_IEXEC;
		}
	}
	return 0;
}
#endif