#ifdef _WIN32_WCE
#include <windows.h>
#include "time.h"

#include "formatParser.h"

static LCTYPE AbbDayName[] = { LOCALE_SABBREVDAYNAME7, LOCALE_SABBREVDAYNAME1, 
                        LOCALE_SABBREVDAYNAME2, LOCALE_SABBREVDAYNAME3, 
						LOCALE_SABBREVDAYNAME4, LOCALE_SABBREVDAYNAME5, 
						LOCALE_SABBREVDAYNAME6 };

static LCTYPE FullDayName[] = { LOCALE_SDAYNAME7, LOCALE_SDAYNAME1, LOCALE_SDAYNAME2, 
                         LOCALE_SDAYNAME3, LOCALE_SDAYNAME4,
						 LOCALE_SDAYNAME5, LOCALE_SDAYNAME6 };
static LCTYPE AbbMonthName[] = { LOCALE_SABBREVMONTHNAME1, LOCALE_SABBREVMONTHNAME2, 
                          LOCALE_SABBREVMONTHNAME3, LOCALE_SABBREVMONTHNAME4,
						  LOCALE_SABBREVMONTHNAME5, LOCALE_SABBREVMONTHNAME6,
						  LOCALE_SABBREVMONTHNAME7, LOCALE_SABBREVMONTHNAME8, 
						  LOCALE_SABBREVMONTHNAME9, LOCALE_SABBREVMONTHNAME10,
						  LOCALE_SABBREVMONTHNAME11, LOCALE_SABBREVMONTHNAME12 };
static LCTYPE FullMonthName[] = { LOCALE_SMONTHNAME1, LOCALE_SMONTHNAME2, 
                           LOCALE_SMONTHNAME3, LOCALE_SMONTHNAME4,
						   LOCALE_SMONTHNAME5, LOCALE_SMONTHNAME6,
						   LOCALE_SMONTHNAME7, LOCALE_SMONTHNAME8, 
						   LOCALE_SMONTHNAME9, LOCALE_SMONTHNAME10,
						   LOCALE_SMONTHNAME11, LOCALE_SMONTHNAME12 };

const int MAXTOKENVALUE_LEN = 100;
const int MAXLOCALEINFO_LEN = 100;

static int GetLocaleInfoA(LCID Locale, LCTYPE LCType, char* lpLCData, int cchData)
{
	int result;
	BOOL bUsed;
	TCHAR wLocaleInfo[MAXLOCALEINFO_LEN];

	result = GetLocaleInfo(Locale, LCType, wLocaleInfo, MAXLOCALEINFO_LEN);
	if (result == 0) return 0;
	result = WideCharToMultiByte(CP_ACP, 0, wLocaleInfo, result, 
	                             lpLCData, cchData, NULL, &bUsed);
	return result;
}

size_t strftime(char *s, size_t maxsize, const char *format, 
				const struct tm *timeptr)
{
	if (timeptr == NULL) return 0;

    CFormatParser parser(format);

	char value[MAXTOKENVALUE_LEN];
	char localeInfo[MAXLOCALEINFO_LEN];
	TCHAR wLocaleInfo[2];

/*	GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, LOCALE_ILZERO, localeInfo, 100);
	bool leadingZero = (*localeInfo == '1') ? true : false;*/
	GetLocaleInfo(LOCALE_SYSTEM_DEFAULT, LOCALE_ILZERO, wLocaleInfo, 2);
	bool leadingZero = (*wLocaleInfo == _T('1')) ? true : false;

	TOKEN t = parser.GetNextToken(value);
	while (t != TOKEN_EMPTY) {
		switch (t) {
			case TOKEN_a:
				GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, AbbDayName[timeptr->tm_wday],
					          localeInfo, MAXLOCALEINFO_LEN);
				strcat(s, localeInfo);
				break;

			case TOKEN_A:
				GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, FullDayName[timeptr->tm_wday],
					          localeInfo, MAXLOCALEINFO_LEN);
				strcat(s, localeInfo);
				break;

			case TOKEN_b:
				GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, AbbMonthName[timeptr->tm_mon],
					          localeInfo, MAXLOCALEINFO_LEN);
				strcat(s, localeInfo);
				break;

			case TOKEN_B:
				GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, FullMonthName[timeptr->tm_mon],
					          localeInfo, MAXLOCALEINFO_LEN);
				strcat(s, localeInfo);
				break;

			case TOKEN_c:
			{
				char dateFormat[2];
				char timeFormat[2];
				char sdate[MAXLOCALEINFO_LEN];
				char stime[MAXLOCALEINFO_LEN];

				GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, LOCALE_IDATE, 
					          dateFormat, 2);
				GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, LOCALE_SDATE,
					          sdate, MAXLOCALEINFO_LEN);

				GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, LOCALE_ITIME, 
					          timeFormat, 2);
				GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, LOCALE_STIME,
					          stime, MAXLOCALEINFO_LEN);

				int year = timeptr->tm_year;
				while (year >= 100) {
					year -= 100;
				}

				switch (*dateFormat) {
					case '0':
						if (leadingZero) 
							sprintf(localeInfo, "%02d%s%02d%s%02d %02d%s%02d%s%02d", 
							        timeptr->tm_mon + 1, sdate, timeptr->tm_mday, sdate, year,
									timeptr->tm_hour, stime, timeptr->tm_min, stime, timeptr->tm_sec);
						else
							sprintf(localeInfo, "%d%s%d%s%d %d%s%d%s%d", 
							        timeptr->tm_mon + 1, sdate, timeptr->tm_mday, sdate, year,
									timeptr->tm_hour, stime, timeptr->tm_min, stime, timeptr->tm_sec);
						break;

					case '1':
						if (leadingZero) 
							sprintf(localeInfo, "%02d%s%02d%s%02d %02d%s%02d%s%02d", 
							        timeptr->tm_mday, sdate, timeptr->tm_mon + 1, sdate, year,
									timeptr->tm_hour, stime, timeptr->tm_min, stime, timeptr->tm_sec);
						else
							sprintf(localeInfo, "%d%s%d%s%d %d%s%d%s%d", 
							        timeptr->tm_mday, sdate, timeptr->tm_mon + 1, sdate, year,
									timeptr->tm_hour, stime, timeptr->tm_min, stime, timeptr->tm_sec);
						break;

					case '2':
						if (leadingZero) 
							sprintf(localeInfo, "%02d%s%02d%s%02d %02d%s%02d%s%02d", 
							        year, sdate, timeptr->tm_mon + 1, sdate, timeptr->tm_mday,
									timeptr->tm_hour, stime, timeptr->tm_min, stime, timeptr->tm_sec);
						else
							sprintf(localeInfo, "%d%s%d%s%d %d%s%d%s%d", 
							        year, sdate, timeptr->tm_mon + 1, sdate, timeptr->tm_mday,
									timeptr->tm_hour, stime, timeptr->tm_min, stime, timeptr->tm_sec);
						break;
				}

				strcat(s, localeInfo);
				break;
			}

			case TOKEN_d:
				if (leadingZero) 
					sprintf(localeInfo, "%02d", timeptr->tm_mday);
				else
					sprintf(localeInfo, "%d", timeptr->tm_mday);
				strcat(s, localeInfo);
				break;

			case TOKEN_H:
				if (leadingZero)
					sprintf(localeInfo, "%02d", timeptr->tm_hour);
				else
					sprintf(localeInfo, "%d", timeptr->tm_hour);
				strcat(s, localeInfo);
				break;

			case TOKEN_I:
			{
				int hour = (timeptr->tm_hour > 12) ? (timeptr->tm_hour - 12) : timeptr->tm_hour;
				if (leadingZero) 
					sprintf(localeInfo, "%02d", hour);
				else
					sprintf(localeInfo, "%d", hour);
				strcat(s, localeInfo);
				break;
			}

			case TOKEN_j:
				if (leadingZero)
					sprintf(localeInfo, "%03d", timeptr->tm_yday + 1);
				else
					sprintf(localeInfo, "%d", timeptr->tm_yday + 1);
				strcat(s, localeInfo);
				break;

			case TOKEN_m:
				if (leadingZero)
					sprintf(localeInfo, "%02d", timeptr->tm_mon + 1);
				else
					sprintf(localeInfo, "%d", timeptr->tm_mon + 1);
				strcat(s, localeInfo);
				break;

			case TOKEN_M:
				if (leadingZero)
					sprintf(localeInfo, "%2d", timeptr->tm_min);
				else
					sprintf(localeInfo, "%d", timeptr->tm_min);
				strcat(s, localeInfo);
				break;

			case TOKEN_p:
				if (timeptr->tm_hour + 1 >= 12 && timeptr->tm_hour <= 23) // PM
					GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, LOCALE_S2359,
								  localeInfo, MAXLOCALEINFO_LEN);
				else // AM
					GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, LOCALE_S1159,
								  localeInfo, MAXLOCALEINFO_LEN);
				strcat(s, localeInfo);
				break;

			case TOKEN_S:
				if (leadingZero) 
					sprintf(localeInfo, "%02d", timeptr->tm_sec);
				else
					sprintf(localeInfo, "%d", timeptr->tm_sec);
				strcat(s, localeInfo);
				break;

			case TOKEN_U:
			{
				int wNumber = timeptr->tm_yday / 7;	
				//int diff = (timeptr->tm_yday + 1) - (wNumber * 7);
				int diff = (timeptr->tm_yday + 1) - (wNumber * 7);
				if (diff >= (timeptr->tm_wday + 1)) wNumber++;

				if (leadingZero) 
					sprintf(localeInfo, "%02d", wNumber);
				else
					sprintf(localeInfo, "%d", wNumber);
				strcat(s, localeInfo);
				break;
			}

			case TOKEN_w:
				sprintf(localeInfo, "%d", timeptr->tm_wday);
				strcat(s, localeInfo);
				break;

			case TOKEN_W:
			{
				int wNumber = timeptr->tm_yday / 7;	
				int diff = (timeptr->tm_yday + 1) - (wNumber * 7);

				if (timeptr->tm_wday >= 1 && diff >= timeptr->tm_wday) wNumber++;
				if (leadingZero) 
					sprintf(localeInfo, "%02d", wNumber);
				else
					sprintf(localeInfo, "%d", wNumber);
				strcat(s, localeInfo);
				break;
			}

			case TOKEN_x:
			{
				char dateFormat[2];
				char timeFormat[2];
				char sdate[MAXLOCALEINFO_LEN];
				char stime[MAXLOCALEINFO_LEN];

				GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, LOCALE_IDATE, 
					          dateFormat, 2);
				GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, LOCALE_SDATE,
					          sdate, MAXLOCALEINFO_LEN);

				GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, LOCALE_ITIME, 
					          timeFormat, 2);
				GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, LOCALE_STIME,
					          stime, MAXLOCALEINFO_LEN);

				int year = timeptr->tm_year;
				while (year >= 100) {
					year -= 100;
				}

				switch (*dateFormat) {
					case '0':
						if (leadingZero) 
							sprintf(localeInfo, "%02d%s%02d%s%02d", 
							        timeptr->tm_mon + 1, sdate, timeptr->tm_mday, sdate, year);
						else
							sprintf(localeInfo, "%d%s%d%s%d", 
							        timeptr->tm_mon + 1, sdate, timeptr->tm_mday, sdate, year);
						break;

					case '1':
						if (leadingZero) 
							sprintf(localeInfo, "%02d%s%02d%s%02d", 
							        timeptr->tm_mday, sdate, timeptr->tm_mon + 1, sdate, year);
						else
							sprintf(localeInfo, "%d%s%d%s%d", 
							        timeptr->tm_mday, sdate, timeptr->tm_mon + 1, sdate, year);
						break;

					case '2':
						if (leadingZero) 
							sprintf(localeInfo, "%02d%s%02d%s%02d", 
							        year, sdate, timeptr->tm_mon + 1, sdate, timeptr->tm_mday);
						else
							sprintf(localeInfo, "%d%s%d%s%d", 
							        year, sdate, timeptr->tm_mon + 1, sdate, timeptr->tm_mday);
						break;
				}

				strcat(s, localeInfo);
				break;
			}

			case TOKEN_X:
			{
				char dateFormat[2];
				char timeFormat[2];
				char sdate[MAXLOCALEINFO_LEN];
				char stime[MAXLOCALEINFO_LEN];

				GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, LOCALE_IDATE, 
					          dateFormat, 2);
				GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, LOCALE_SDATE,
					          sdate, MAXLOCALEINFO_LEN);

				GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, LOCALE_ITIME, 
					          timeFormat, 2);
				GetLocaleInfoA(LOCALE_SYSTEM_DEFAULT, LOCALE_STIME,
					          stime, MAXLOCALEINFO_LEN);

				switch (*dateFormat) {
					case '0':
						if (leadingZero) 
							sprintf(localeInfo, "%02d%s%02d%s%02d", 
							        timeptr->tm_hour, stime, timeptr->tm_min, stime, timeptr->tm_sec);
						else
							sprintf(localeInfo, "%d%s%d%s%d", 
									timeptr->tm_hour, stime, timeptr->tm_min, stime, timeptr->tm_sec);
						break;

					case '1':
						if (leadingZero) 
							sprintf(localeInfo, "%02d%s%02d%s%02d", 
									timeptr->tm_hour, stime, timeptr->tm_min, stime, timeptr->tm_sec);
						else
							sprintf(localeInfo, "%d%s%d%s%d", 
									timeptr->tm_hour, stime, timeptr->tm_min, stime, timeptr->tm_sec);
						break;

					case '2':
						if (leadingZero) 
							sprintf(localeInfo, "%02d%s%02d%s%02d", 
									timeptr->tm_hour, stime, timeptr->tm_min, stime, timeptr->tm_sec);
						else
							sprintf(localeInfo, "%d%s%d%s%d", 
									timeptr->tm_hour, stime, timeptr->tm_min, stime, timeptr->tm_sec);
						break;
				}

				strcat(s, localeInfo);
				break;
			}

			case TOKEN_y:
			{
				int year = timeptr->tm_year;

				while (year >= 100) {
					year -= 100;
				}
				if (leadingZero)
					sprintf(localeInfo, "%02d", year);
				else
					sprintf(localeInfo, "%d", year);
				strcat(s, localeInfo);
				break;
			}

			case TOKEN_Y:
				sprintf(localeInfo, "%d", timeptr->tm_year + 1900);
				strcat(s, localeInfo);
				break;

			case TOKEN_z:
				TIME_ZONE_INFORMATION t;

				GetTimeZoneInformation(&t);
				WideCharToMultiByte(CP_ACP, 0, t.StandardName, 
					                -1, localeInfo, MAXLOCALEINFO_LEN, NULL, NULL);
				strcat(s, localeInfo);
				break;

			case TOKEN_PERCENT:
				strcat(s, "%");
				break;

			case TOKEN_ERROR:
				break;

			case TOKEN_VALUE:
			{
				strcat(s, value);
				break;
			}
		}
		t = parser.GetNextToken(value);
	}
	return strlen(s);
}
#endif