#ifdef _WIN32_WCE
#include <windows.h>
#include "time.h"
#include "sys\timeb.h"

const __int64 OneSecIn100NanoSec = 10000000;
const __int64 OneMilliSecIn100NanoSec = 10000;

static FILETIME baseFileTime;
static struct tm gTime;

static FILETIME YearToFileTime(WORD wYear)
{	
	SYSTEMTIME systemTime;
	FILETIME fileTime;

	systemTime.wYear = wYear;
	systemTime.wMonth = 1;
	systemTime.wDayOfWeek = 1; 
	systemTime.wDay = 1;
	systemTime.wHour = 0;
	systemTime.wMinute = 0;
	systemTime.wSecond = 0;
	systemTime.wMilliseconds = 0;

	SystemTimeToFileTime(&systemTime, &fileTime);

	return fileTime;
}

static __int64 GetDifferenceBetweenFileTime(const FILETIME *first, const FILETIME *second)
{
	__int64 f, s;

	f = first->dwHighDateTime;
	f <<= 32;
	f |= first->dwLowDateTime;

	s = second->dwHighDateTime;
	s <<= 32;
	s |= second->dwLowDateTime;

	if (f >= s) return f - s;
	else return s - f;
}

static void SubtractFileTime(const FILETIME *first, const FILETIME *second, FILETIME* result)
{
	__int64 f, s, r;

	f = first->dwHighDateTime;
	f <<= 32;
	f |= first->dwLowDateTime;

	s = second->dwHighDateTime;
	s <<= 32;
	s |= second->dwLowDateTime;

	r = f - s;

	result->dwHighDateTime = (DWORD) ((r >> 32) & 0x00000000FFFFFFFF);
	result->dwLowDateTime = (DWORD) (r & 0x00000000FFFFFFFF);
}

static void AddFileTime(const FILETIME *first, const FILETIME *second, FILETIME *result)
{
	__int64 f, s, r;

	f = first->dwHighDateTime;
	f <<= 32;
	f |= first->dwLowDateTime;

	s = second->dwHighDateTime;
	s <<= 32;
	s |= second->dwLowDateTime;

	r = f + s;

	result->dwHighDateTime = (DWORD) ((r >> 32) & 0x00000000FFFFFFFF);
	result->dwLowDateTime = (DWORD) (r & 0x00000000FFFFFFFF);
}

static void SystemTimeToTm(const SYSTEMTIME* systemTime, struct tm *time)
{
	FILETIME fileTime, baseTime;
	__int64 diff;

	if (systemTime && time) {
		time->tm_year = systemTime->wYear - 1900;
		time->tm_hour = systemTime->wHour;
		time->tm_min = systemTime->wMinute;
		time->tm_mon = systemTime->wMonth;
		time->tm_sec = systemTime->wSecond;
		time->tm_wday = systemTime->wDayOfWeek;
		time->tm_mday = systemTime->wDay;

		SystemTimeToFileTime(systemTime, &fileTime);
		baseTime = YearToFileTime(systemTime->wYear);
		diff = GetDifferenceBetweenFileTime(&fileTime, &baseTime) / OneSecIn100NanoSec;
		time->tm_yday = (int) (diff / (__int64) (24 * 3600));

		time->tm_isdst = 0;  // will be ignored.
	}
}

static void TmToSystemTime(const struct tm *time, SYSTEMTIME* systemTime)
{
	if (time && systemTime) {
		systemTime->wYear = time->tm_year + 1900;
		systemTime->wHour = time->tm_hour;
		systemTime->wMinute = time->tm_min;
		systemTime->wMonth = time->tm_mon;
		systemTime->wSecond = time->tm_sec;
		systemTime->wDayOfWeek = time->tm_wday;
		systemTime->wDay = time->tm_mday;
		systemTime->wMilliseconds = 0;
	}
}

void InitTime(void)
{
	baseFileTime = YearToFileTime(1970);
}

time_t GetFileTimeAfter1970(FILETIME* fileTime) 
{
	return (time_t) (GetDifferenceBetweenFileTime(fileTime, &baseFileTime) / OneSecIn100NanoSec);
}

struct tm *gmtime(const time_t *timer)
{
	FILETIME time;
	FILETIME newTime;
	SYSTEMTIME systemTime;
	__int64 timeInSecs = *timer;

	timeInSecs *= OneSecIn100NanoSec; // convert to FILETIME format

	time.dwLowDateTime = (DWORD) (timeInSecs & 0x00000000FFFFFFFF);
	time.dwHighDateTime = (DWORD) ((timeInSecs >> 32) & 0x00000000FFFFFFFF);

	AddFileTime(&time, &baseFileTime, &newTime);

	FileTimeToSystemTime(&newTime, &systemTime);
	SystemTimeToTm(&systemTime, &gTime);

	return &gTime;
}

void _ftime(struct _timeb *timeptr)
{	
    SYSTEMTIME systemTime;
    FILETIME fileTime;
	ULONGLONG diff;
	ULONGLONG diffInSec;

	GetSystemTime(&systemTime);
	SystemTimeToFileTime(&systemTime, &fileTime);
	diff = GetDifferenceBetweenFileTime(&fileTime, &baseFileTime);

	timeptr->dstflag = 0; //ignored
	timeptr->timezone = 0; // ignored
	diffInSec = (diff / OneSecIn100NanoSec);
	timeptr->time = (time_t) diffInSec;
	timeptr->millitm = (unsigned short) (time_t) (diff / OneMilliSecIn100NanoSec) - (diffInSec * 1000);
}

#endif
